@@ -42,6 +42,8 @@ namespace Opm {
 class OpmLog {
 
 public:
+    constexpr static int defaultDebugVerbosityLevel = 1;
+
     static void addMessage(int64_t messageFlag , const std::string& message);
     static void addTaggedMessage(int64_t messageFlag, const std::string& tag, const std::string& message);
 
@@ -50,7 +52,7 @@ public:
     static void error(const std::string& message);
     static void problem(const std::string& message);
     static void bug(const std::string& message);
-    static void debug(const std::string& message);
+    static void debug(const std::string& message, const int verbosity_level = defaultDebugVerbosityLevel);
     static void note(const std::string& message);
 
     static void info(const std::string& tag, const std::string& message);
@@ -68,6 +70,10 @@ public:
     static bool enabledMessageType( int64_t messageType );
     static void addMessageType( int64_t messageType , const std::string& prefix);
 
+
+    static int getDebugVerbosityLevel() { return debug_verbosity_level_; }
+    static void setDebugVerbosityLevel(const int verbosity_level) { debug_verbosity_level_ = verbosity_level; }
+
     /// Create a basic logging setup that will send all log messages to standard output.
     ///
     /// By default category prefixes will be printed (i.e. Error: or
@@ -113,6 +119,7 @@ private:
 #ifdef EMBEDDED_PYTHON
     friend class PyRunModule;
 #endif
+    static int debug_verbosity_level_;
     static std::shared_ptr<Logger> getLogger();
     static std::shared_ptr<Logger> m_logger;
 };
