@@ -48,11 +48,11 @@ namespace Opm {
 	public:
 	    /// @brief
 	    /// @todo Doc me!
-	    virtual ~Parameter() {}
+            ~Parameter() override {}
 	    /// @brief
 	    /// @todo Doc me!
 	    /// @return
-	    virtual std::string getTag() const {return ID_xmltag__param;}
+            std::string getTag() const override { return ID_xmltag__param; }
 	    /// @brief
 	    /// @todo Doc me!
 	    /// @param
@@ -61,11 +61,12 @@ namespace Opm {
 	    /// @brief
 	    /// @todo Doc me!
 	    /// @return
-	    std::string getValue() const {return value_;}
+            const std::string& getValue() const { return value_; }
 	    /// @brief
 	    /// @todo Doc me!
 	    /// @return
-	    std::string getType() const {return type_;}
+            const std::string& getType() const { return type_; }
+
 	private:
 	    std::string value_;
 	    std::string type_;
