@@ -38,6 +38,14 @@ namespace Opm {
     {
     }
 
+    DeckKeyword::DeckKeyword(const KeywordLocation& location, const ParserKeyword& parserKeyword) :
+        m_keywordName(parserKeyword.getName()),
+        m_location(location),
+        m_isDataKeyword(false),
+        m_slashTerminated(!parserKeyword.hasFixedSize())
+    {
+    }
+
     DeckKeyword::DeckKeyword(const KeywordLocation& location, const std::string& keywordName) :
         m_keywordName(keywordName),
         m_location(location),
@@ -65,6 +73,15 @@ namespace Opm {
         return result;
     }
 
+    DeckKeyword DeckKeyword::emptyStructuralCopy() const
+    {
+        auto ret = *this;
+
+        ret.m_recordList.clear();
+
+        return ret;
+    }
+
     namespace {
     template <typename T>
     void add_deckvalue( DeckItem deck_item, DeckRecord& deck_record, const ParserItem& parser_item, const std::vector<DeckValue>& input_record, size_t j) {
@@ -154,8 +171,8 @@ namespace Opm {
         }
     }
 
-    DeckKeyword::DeckKeyword(const ParserKeyword& parserKeyword, const std::vector<int>& data) :
-        DeckKeyword(parserKeyword)
+    DeckKeyword::DeckKeyword(const ParserKeyword& parserKeyword, const std::vector<int>& data, const KeywordLocation& location) :
+        DeckKeyword(location, parserKeyword)
     {
         if (!parserKeyword.isDataKeyword())
             throw std::invalid_argument("Deckkeyword '" + name() + "' is not a data keyword.");
@@ -176,8 +193,8 @@ namespace Opm {
     }
 
 
-    DeckKeyword::DeckKeyword(const ParserKeyword& parserKeyword, const std::vector<double>& data, const UnitSystem& system_active, const UnitSystem& system_default) :
-        DeckKeyword(parserKeyword)
+    DeckKeyword::DeckKeyword(const ParserKeyword& parserKeyword, const std::vector<double>& data, const UnitSystem& system_active, const UnitSystem& system_default, const KeywordLocation& location) :
+        DeckKeyword(location, parserKeyword)
     {
         if (!parserKeyword.isDataKeyword())
             throw std::invalid_argument("Deckkeyword '" + name() + "' is not a data keyword.");
@@ -282,16 +299,25 @@ namespace Opm {
         return this->getDataRecord().getDataItem().data_size();
     }
 
-
-    const std::vector<int>& DeckKeyword::getIntData() const {
-        return this->getDataRecord().getDataItem().getData< int >();
+    std::vector<int>& DeckKeyword::getIntData()
+    {
+         return this->getRecord(0).getItem(0).getData<int>();
     }
 
-
-    const std::vector<std::string>& DeckKeyword::getStringData() const {
-        return this->getDataRecord().getDataItem().getData< std::string >();
+    const std::vector<int>& DeckKeyword::getIntData() const
+    {
+        return this->getDataRecord().getDataItem().getData<int>();
     }
 
+    const std::vector<std::string>& DeckKeyword::getStringData() const
+    {
+        return this->getDataRecord().getDataItem().getData<std::string>();
+    }
+
+    std::vector<double>& DeckKeyword::getRawDoubleData()
+    {
+        return this->getRecord(0).getItem(0).getData<double>();
+    }
 
     const std::vector<double>& DeckKeyword::getRawDoubleData() const {
         return this->getDataRecord().getDataItem().getData< double >();
@@ -371,4 +397,3 @@ namespace Opm {
     }
 
 }
-
