@@ -133,7 +133,7 @@ namespace Opm {
     try
         : m_tables(            deck )
         , m_runspec(           deck )
-        , m_eclipseConfig(     deck )
+        , m_eclipseConfig(     deck, m_runspec.phases() )
         , m_deckUnitSystem(    deck.getActiveUnitSystem() )
         , m_inputGrid(         deck, nullptr )
         , m_inputNnc(          m_inputGrid, deck)
@@ -502,7 +502,7 @@ namespace Opm {
                 }
                 catch (const std::exception& std_error)
                 {
-                    OpmLog::error(fmt::format("\nMULTFLT: Cannot set fault transmissibility multiplier\n" 
+                    OpmLog::error(fmt::format("\nMULTFLT: Cannot set fault transmissibility multiplier\n"
                        "MULTFLT(FLTNAME) equals {} and MULT(FLT-TRS) equals {}\n"
                        "Error creating reservoir properties: {}" , faultPattern, multFlt, std_error.what()));
                     error = true;
