@@ -59,12 +59,13 @@ namespace Opm {
         std::unique_ptr<Carfin> m_keywordCarfin;
 
         std::unique_ptr<Carfin>
-        makeLgr(std::string name, int i1, int i2, 
-                int j1, int j2, 
-                int k1, int k2, 
-                int nx , int ny , int nz) const;
+        makeLgr(const std::string& name,
+                int i1, int i2,
+                int j1, int j2,
+                int k1, int k2,
+                int nx, int ny, int nz) const;
     };
 }
 
 
-#endif  
+#endif
