@@ -17,15 +17,15 @@
   along with OPM.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-
-
 #ifndef OPM_PARSER_ECLIPSE_GRID_HPP
 #define OPM_PARSER_ECLIPSE_GRID_HPP
+
 #include <opm/input/eclipse/EclipseState/Grid/GridDims.hpp>
 #include <opm/input/eclipse/EclipseState/Grid/MapAxes.hpp>
 #include <opm/input/eclipse/EclipseState/Grid/MinpvMode.hpp>
 #include <opm/input/eclipse/EclipseState/Grid/PinchMode.hpp>
 
+#include <algorithm>
 #include <array>
 #include <memory>
 #include <optional>
@@ -35,7 +35,6 @@
 #include <vector>
 #include <map>
 
-
 namespace Opm {
 
     class Deck;
@@ -109,6 +108,24 @@ namespace Opm {
         size_t getActiveIndex(size_t i, size_t j, size_t k) const {
             return activeIndex(i, j, k);
         }
+        const std::vector<std::size_t>& get_print_order_lgr () const {
+          return m_print_order_lgr_cells;
+        }
+
+        size_t get_lgr_cell_index(const std::string& lgr_tag) const {
+          const auto& labels = get_all_lgr_labels();
+
+          if (labels.empty()) {
+              throw std::runtime_error("No LGR labels available.");
+          }
+
+          auto it = std::find(labels.begin(), labels.end(), lgr_tag);
+          if (it == labels.end()) {
+              throw std::runtime_error("LGR tag not found: " + lgr_tag);
+          }
+
+          return static_cast<size_t>(std::distance(labels.begin(), it));
+        }
 
         size_t getActiveIndex(size_t globalIndex) const {
             return activeIndex(globalIndex);
@@ -116,21 +133,30 @@ namespace Opm {
 
         std::vector<std::string> get_all_lgr_labels() const
         {
-            return  {this->all_lgr_labels.begin() + 1, this->all_lgr_labels.end()};
+            if (this->all_lgr_labels.empty()) {
+              return {};
+            } else {
+              return {this->all_lgr_labels.begin() + 1, this->all_lgr_labels.end()};
+            }
+        }
+
+        const std::string& get_lgr_labels_by_number(std::size_t num) const
+        {
+          return  all_lgr_labels[num];
         }
 
         const std::vector<std::string>& get_all_labels() const
         {
             return this->all_lgr_labels;
         }
-        
-        std::string get_lgr_tag() const {
+
+        const std::string& get_lgr_tag() const {
             return this->lgr_label;
         }
 
         std::vector<GridDims> get_lgr_children_gridim() const;
-      
-        
+
+
         void assertIndexLGR(size_t localIndex) const;
 
         void assertLabelLGR(const std::string& label) const;
@@ -145,7 +171,8 @@ namespace Opm {
         void init_children_host_cells(bool logical = true);
         void init_children_host_cells_logical(void);
         void init_children_host_cells_geometrical(void);
-
+        std::array<int,3> getCellSubdivisionRatioLGR(const std::string&  lgr_tag,
+                                                     std::array<int,3>   acum = {1,1,1}) const;
         using GridDims::getGlobalIndex;
         size_t getGlobalIndex(size_t active_index) const;
 
@@ -225,9 +252,9 @@ namespace Opm {
         std::array<double, 3> getCellDimensions(size_t i, size_t j, size_t k) const {
             return getCellDims(i, j, k);
         }
-        std::array<double,3> getCellDimensionsLGR(const std::size_t  i, 
-                                                  const std::size_t  j, 
-                                                  const std::size_t  k, 
+        std::array<double,3> getCellDimensionsLGR(const std::size_t  i,
+                                                  const std::size_t  j,
+                                                  const std::size_t  k,
                                                   const std::string& lgr_tag) const;
         double getCellDepthLGR(size_t i, size_t j, size_t k, const std::string& lgr_tag) const;
 
@@ -287,9 +314,12 @@ namespace Opm {
         static bool hasEqualDVDEPTHZ(const Deck&);
         static bool allEqual(const std::vector<double> &v);
         EclipseGridLGR& getLGRCell(std::size_t index);
+        const EclipseGridLGR& getLGRCell(std::size_t index) const;
         const EclipseGridLGR& getLGRCell(const std::string& lgr_tag) const;
         int getLGR_global_father(std::size_t global_index,  const std::string& lgr_tag) const;
-
+        int getLGR_father(std::size_t i, std::size_t j, std::size_t k, const std::string& lgr_tag) const;
+        int getLGR_father(std::size_t global_index, const std::string& lgr_tag) const;
+        std::array<int,3> getLGR_fatherIJK(std::size_t i, std::size_t j, std::size_t k, const std::string& lgr_tag) const;
         std::vector<EclipseGridLGR> lgr_children_cells;
         /**
         * @brief Sets Local Grid Refinement for the EclipseGrid.
@@ -399,10 +429,7 @@ namespace Opm {
                             std::array<double,8>& X,
                             std::array<double,8>& Y,
                             std::array<double,8>& Z) const;
-        std::array<int,3> getCellSubdivisionRatioLGR(const std::string&  lgr_tag, 
-                                                     std::array<int,3>   acum = {1,1,1}) const;
-    
-    
+
     };
 
     /// Specialized Class to describe LGR refined cells.
@@ -434,11 +461,16 @@ namespace Opm {
         get_child_LGR_cell(const std::string& lgr_tag) const;
         std::vector<int> save_hostnum(void) const;
         int get_hostnum(std::size_t global_index) const {return(m_hostnum[global_index]);};
+
+        //parsing the father grid allows the global_father references to be given in terms of father_grid
+        std::vector<int> getLGRCell_global_father(const EclipseGrid& father_grid) const;
+        std::vector<double> getLGRCell_all_depth (const EclipseGrid& father_grid) const;
+
         void get_label_child_to_top_father(std::vector<std::reference_wrapper<const std::string>>& list) const;
-        void get_global_index_child_to_top_father(std::vector<std::size_t> & list, 
+        void get_global_index_child_to_top_father(std::vector<std::size_t> & list,
                                                   std::size_t i, std::size_t j, std::size_t k) const;
         void get_global_index_child_to_top_father(std::vector<std::size_t> & list, std::size_t global_ind) const;
-  
+
         void set_hostnum(const std::vector<int>&);
         const std::array<int,3>& get_low_fatherIJK() const{
           return low_fatherIJK;
@@ -456,7 +488,7 @@ namespace Opm {
          * @brief Sets Local Grid Refinement for the EclipseGridLGR.
          *
          * @param lgr_tag The string that contains the name of a given LGR cell.
-         * @param coords The coordinates of a given LGR cell in  CPG COORDSformat.
+         * @param coord The coordinates of a given LGR cell in  CPG COORDSformat.
          * @param zcorn The z-coordinates values of a given LGR cell in CPG ZCORN format.
          */
         void set_lgr_refinement(const std::string& lgr_tag,
