@@ -38,6 +38,10 @@
 #include <opm/input/eclipse/EclipseState/Grid/MULTREGTScanner.hpp>
 
 namespace Opm {
+    namespace data {
+        class Solution;
+    }
+
     template< typename > class GridProperty;
     class Fault;
     class FaultCollection;
@@ -62,6 +66,14 @@ namespace Opm {
         void applyMULTFLT(const Fault& fault);
         void applyNumericalAquifer(const std::vector<std::size_t>& aquifer_cells);
 
+        /// \brief Creates a solution object with all multipliers for output
+        /// \param active_cells If the model has no multipliers then this number is used as the size of
+        ///                     the array (containing 1) that are constructed in this case.
+        /// \param include_all_multminus If false only non-defaulted MULT?- arrays will
+        ///                              included. Otherwise even defaulted ones.
+        data::Solution convertToSimProps(std::size_t active_cells,
+                                         bool include_all_multminus) const;
+
         bool operator==(const TransMult& data) const;
 
         template<class Serializer>
