@@ -7,6 +7,8 @@
 namespace Opm {
     class DeckKeyword;
     class DeckRecord;
+    class KeywordLocation;
+    class Phases;
 
     class EquilRecord {
         public:
@@ -18,7 +20,7 @@ namespace Opm {
                         bool wet_gas_init,
                         int target_accuracy,
                         bool humid_gas_init);
-            explicit EquilRecord(const DeckRecord& record);
+            EquilRecord(const DeckRecord& record, const Phases& phases, int region, const KeywordLocation& location);
 
             static EquilRecord serializationTestObject();
             double datumDepth() const;
@@ -67,7 +69,7 @@ namespace Opm {
     class StressEquilRecord {
         public:
             StressEquilRecord() = default;
-            explicit StressEquilRecord(const DeckRecord& record);
+            StressEquilRecord(const DeckRecord& record, const Phases& phases, int region, const KeywordLocation& location);
 
             static StressEquilRecord serializationTestObject();
 
@@ -83,6 +85,13 @@ namespace Opm {
             double stressZZ() const;
             double stressZZ_grad() const;
 
+            double stressXY() const;
+            double stressXY_grad() const;
+            double stressXZ() const;
+            double stressXZ_grad() const;
+            double stressYZ() const;
+            double stressYZ_grad() const;
+
             template<class Serializer>
             void serializeOp(Serializer& serializer)
             {
@@ -95,6 +104,13 @@ namespace Opm {
                 serializer(stress_yy_grad);
                 serializer(stress_zz);
                 serializer(stress_zz_grad);
+
+                serializer(stress_xy);
+                serializer(stress_xy_grad);
+                serializer(stress_xz);
+                serializer(stress_xz_grad);
+                serializer(stress_yz);
+                serializer(stress_yz_grad);
             }
 
         private:
@@ -107,6 +123,13 @@ namespace Opm {
             double stress_yy_grad = 0.0;
             double stress_zz = 0.0;
             double stress_zz_grad = 0.0;
+
+            double stress_xy = 0.0;
+            double stress_xy_grad = 0.0;
+            double stress_xz = 0.0;
+            double stress_xz_grad = 0.0;
+            double stress_yz = 0.0;
+            double stress_yz_grad = 0.0;
     };
 
     template<class RecordType>
@@ -115,7 +138,7 @@ namespace Opm {
             using const_iterator = typename std::vector<RecordType>::const_iterator;
 
             EquilContainer() = default;
-            explicit EquilContainer( const DeckKeyword& );
+            EquilContainer( const DeckKeyword&, const Phases& );
 
             static EquilContainer serializationTestObject();
 
