@@ -336,6 +336,11 @@ public:
      */
     bool activeWag() const;
 
+    /*!
+     * \brief Do Pc scaling for scanning curves.
+     */
+    bool doPcScaling() const;
+
     bool operator==(const EclHysterConfig& data) const;
 
     template<class Serializer>
@@ -347,6 +352,7 @@ public:
         serializer(modParamTrappedValue);
         serializer(curvatureCapPrsValue);
         serializer(activeWagHyst);
+        serializer(enablePcScaling);
     }
 
 private:
@@ -363,6 +369,9 @@ private:
 
     // enable WAG hysteresis
     bool activeWagHyst  { false };
+
+    // flag to enable Pc scaling
+    bool enablePcScaling { false };
 };
 
 class SatFuncControls {
@@ -508,8 +517,10 @@ public:
     bool h2Storage() const noexcept;
     bool micp() const noexcept;
     bool mech() const noexcept;
+    bool frac() const noexcept;
     bool temp() const noexcept;
     bool compositional() const noexcept;
+    bool biof() const noexcept;
 
     bool operator==(const Runspec& data) const;
     static bool rst_cmp(const Runspec& full_state, const Runspec& rst_state);
@@ -539,7 +550,9 @@ public:
         serializer(m_h2storage);
         serializer(m_micp);
         serializer(m_mech);
+        serializer(m_frac);
         serializer(m_temp);
+        serializer(m_biof);
     }
 
 private:
@@ -565,7 +578,9 @@ private:
     bool m_h2storage{false};
     bool m_micp{false};
     bool m_mech{false};
+    bool m_frac{false};
     bool m_temp{false};
+    bool m_biof{false};
 };
 
 std::size_t declaredMaxRegionID(const Runspec& rspec);
