@@ -30,8 +30,8 @@ namespace Opm {
     public:
         PvtgwTable() = default;
         PvtgwTable( const DeckKeyword& keyword, size_t tableIdx);
-        
-        static PvtgwTable serializeObject();
+
+        static PvtgwTable serializationTestObject();
 
         bool operator==(const PvtgwTable& data) const;
     };
