@@ -19,10 +19,14 @@
 
 #include <opm/input/eclipse/Parser/ErrorGuard.hpp>
 
-#include <iostream>
+#include <algorithm>
+#include <cstdlib>
 #include <iomanip>
+#include <iostream>
 #include <numeric>
 
+#include <fmt/format.h>
+
 namespace Opm {
 
     void ErrorGuard::addError(const std::string& errorKey, const std::string& msg) {
@@ -36,14 +40,7 @@ namespace Opm {
 
 
     void ErrorGuard::dump() const {
-        auto maxit = [](const auto acc, const auto& pair)
-                     {
-                         return std::max(acc, pair.first.size());
-                     };
-        std::size_t width = std::accumulate(this->warning_list.begin(),
-                                            this->warning_list.end(), 0UL, maxit);
-        width = std::accumulate(this->error_list.begin(),
-                                this->error_list.end(), width, maxit);
+        const auto width = maxMessageWidth();
 
         if (!this->warning_list.empty()) {
             std::cerr << "Warnings:" << std::endl;
@@ -54,13 +51,39 @@ namespace Opm {
 
         if (!this->error_list.empty()) {
             std::cerr << std::endl << std::endl << "Errors:" << std::endl;
-            for (const auto& pair : this->error_list)
-                std::cerr << std::left << "  " << std::setw(width) << pair.first << ": " << pair.second << std::endl;
+
+            for (const auto& [error_key, error_msg] : this->error_list) {
+                std::cerr << std::left << "  " << std::setw(width) << error_key << ": " << error_msg << std::endl;
+            }
             std::cerr << std::endl;
         }
     }
 
 
+    std::string ErrorGuard::formattedErrors() const {
+        // format error messages to be logged before exiting
+        std::string error_msgs;
+        if (!this->error_list.empty()) {
+            for (const auto& [error_key, error_msg] : this->error_list) {
+                error_msgs += fmt::format("\n{}", error_msg);
+            }
+        }
+        return error_msgs;
+    }
+
+
+    std::size_t ErrorGuard::maxMessageWidth() const {
+        auto maxit = [](const std::size_t acc, const auto& pair)
+                     {
+                         return std::max(acc, pair.first.size());
+                     };
+        std::size_t width = std::accumulate(this->warning_list.begin(),
+                                            this->warning_list.end(), 0UL, maxit);
+        width = std::accumulate(this->error_list.begin(),
+                                this->error_list.end(), width, maxit);
+        return width;
+    }
+
     void ErrorGuard::clear() {
         this->warning_list.clear();
         this->error_list.clear();
