@@ -43,10 +43,12 @@ bool PyAction::valid_keyword(const std::string& keyword) {
         "COMPSEGS",
         "FIELD",
         "ENDBOX", "EXIT",
-        "GCONINJE", "GCONPROD", "GCONSUMP","GRUPTREE",
+        //INCLUDE is allowed as well, but is handled differently by the Parser and thus does not need to be in this list
+        "GCONINJE", "GCONPROD", "GCONSUMP", "GEFAC", "GRUPTREE",
         "METRIC", "MULTX", "MULTX-", "MULTY", "MULTY-", "MULTZ", "MULTZ-",
         "NEXT", "NEXTSTEP",
-        "WCONINJE", "WCONPROD", "WECON", "WEFAC", "WELOPEN", "WELTARG", "WGRUPCON", "WELSEGS", "WELSPECS", "WSEGVALV", "WTEST"
+        "WCONINJE", "WCONPROD", "WECON", "WEFAC", "WELOPEN", "WELPI", "WELTARG", "WGRUPCON",
+        "WELSEGS", "WELSPECS", "WSEGVALV", "WLIST", "WPIMULT", "WTEST", "WTMULT"
     };
     return pyaction_allowed_list.find(keyword) != pyaction_allowed_list.end();
 }
@@ -109,7 +111,8 @@ bool PyAction::operator==(const PyAction& other) const {
 #ifndef EMBEDDED_PYTHON
 
 bool PyAction::run(EclipseState&, Schedule&, std::size_t, SummaryState&,
-                   const std::function<void(const std::string&, const std::vector<std::string>&)>&) const
+                   const std::function<void(const std::string&, const std::vector<std::string>&)>&,
+                   const std::unordered_map<std::string, double>&) const
 {
     return false;
 }
@@ -131,8 +134,12 @@ PyAction::PyAction(std::shared_ptr<const Python> python, const std::string& name
 }
 
 
-bool PyAction::run(EclipseState& ecl_state, Schedule& schedule, std::size_t report_step, SummaryState& st,
-                   const std::function<void(const std::string&, const std::vector<std::string>&)>& actionx_callback) const
+bool PyAction::run(EclipseState& ecl_state,
+                   Schedule& schedule,
+                   std::size_t report_step,
+                   SummaryState& st,
+                   const std::function<void(const std::string&, const std::vector<std::string>&)>& actionx_callback,
+                   const std::unordered_map<std::string, double>& target_wellpi) const
 {
     /*
       For PyAction instances which have been constructed the 'normal' way
@@ -145,7 +152,7 @@ bool PyAction::run(EclipseState& ecl_state, Schedule& schedule, std::size_t repo
     if (!this->run_module)
         this->run_module = std::make_shared<Opm::PyRunModule>(schedule.python(), this->module_file);
 
-    return this->run_module->run(ecl_state, schedule, report_step, st, actionx_callback);
+    return this->run_module->run(ecl_state, schedule, report_step, st, actionx_callback, target_wellpi);
 }
 
 
