@@ -34,7 +34,7 @@
 namespace Opm {
 
 class Schedule;
-enum class WellGuideRateTarget;
+enum class WellGuideRateTarget : std::uint8_t;
 
 } // namespace Opm
 
@@ -203,11 +203,10 @@ private:
                        const double       wat_pot);
 
     double eval_form(const GuideRateModel& model,
+                     const std::string&    wgId,
                      const double          oil_pot,
                      const double          gas_pot,
                      const double          wat_pot) const;
-    double eval_group_pot() const;
-    double eval_group_resvinj() const;
 
     void assign_grvalue(const std::string&    wgname,
                         const GuideRateModel& model,
