@@ -30,6 +30,7 @@
 #include <opm/input/eclipse/Schedule/Action/SimulatorUpdate.hpp>
 #include <opm/input/eclipse/Schedule/Schedule.hpp>
 #include <opm/input/eclipse/Schedule/Well/Well.hpp>
+#include <opm/input/eclipse/Schedule/Well/WellMatcher.hpp>
 
 #include "MSW/WelSegsSet.hpp"
 
@@ -94,7 +95,7 @@ void HandlerContext::setExitCode(int code)
 
 bool HandlerContext::updateWellStatus(const std::string& well,
                                       WellStatus status,
-                                      std::optional<KeywordLocation> location)
+                                      const std::optional<KeywordLocation>& location)
 {
     return schedule_.updateWellStatus(well, currentStep,
                                       status, location);
@@ -151,6 +152,21 @@ const Action::WGNames& HandlerContext::action_wgnames() const
     return schedule_.action_wgnames;
 }
 
+bool HandlerContext::hasWell(const std::string& pattern) const
+{
+    const auto& schedState = this->schedule_.back();
+
+    return WellMatcher {
+        &schedState.well_order(), // <- Note: Pointer to NameOrder
+        schedState.wlist_manager()
+    }.hasWell(pattern);
+}
+
+bool HandlerContext::hasGroup(const std::string& pattern) const
+{
+    return this->schedule_.back().group_order().anyGroupMatches(pattern);
+}
+
 std::vector<std::string>
 HandlerContext::groupNames(const std::string& pattern) const
 {
