@@ -90,7 +90,7 @@ struct Record {
         const double segment_depth = segment.depth();
         const double segment_distance = segment.totalLength();
 
-        // Using top segment depth may lead to depths outside of the 
+        // Using top segment depth may lead to depths outside of the
         // perforated grid cell, so simply stick to grid cell center in this case
         if (segment_number == 1) {
             center_depth = -1.0;
@@ -174,7 +174,7 @@ namespace {
                 }
 
                 compseg.segment_number = segment_number;
-            
+
             }
 
             // when depth is default or zero, we obtain the depth of the connection based on the information
@@ -346,7 +346,6 @@ namespace {
                     throw std::runtime_error("Not all the connections are attached with a segment. "
                                              "The information from COMPSEGS is not complete");
             }
-            new_segment_set.updatePerfLength( new_connection_set );
 
             return std::make_pair( WellConnections( std::move( new_connection_set ) ),
                                    WellSegments( std::move(new_segment_set)));
@@ -421,7 +420,6 @@ namespace {
         auto comp_pressure_drop = pressure_drop_from_int(rst_well.msw_pressure_drop_model);
 
         WellSegments segments( comp_pressure_drop, segments_list);
-        segments.updatePerfLength( connections );
 
         return std::make_pair( std::move(connections), std::move(segments));
     }
