@@ -61,8 +61,8 @@ namespace Opm {
         Segment(const int segment_number_in,
                 const int branch_in,
                 const int outlet_segment_in,
-                const double length_in,
                 const double depth_in,
+                const double length_in,
                 const double internal_diameter_in,
                 const double roughness_in,
                 const double cross_area_in,
@@ -78,7 +78,6 @@ namespace Opm {
         int segmentNumber() const;
         int branchNumber() const;
         int outletSegment() const;
-        double perfLength() const;
         double totalLength() const;
         double node_X() const;
         double node_Y() const;
@@ -103,7 +102,6 @@ namespace Opm {
         const AutoICD& autoICD() const;
         const Valve& valve() const;
 
-        void updatePerfLength(double perf_length);
         void updateSpiralICD(const SICD& spiral_icd);
         void updateAutoICD(const AutoICD& aicd);
         void updateValve(const Valve& valve, const double segment_length);
@@ -137,8 +135,8 @@ namespace Opm {
             serializer(m_branch);
             serializer(m_outlet_segment);
             serializer(m_inlet_segments);
-            serializer(m_total_length);
             serializer(m_depth);
+            serializer(m_total_length);
             serializer(m_internal_diameter);
             serializer(m_roughness);
             serializer(m_cross_area);
@@ -146,7 +144,6 @@ namespace Opm {
             serializer(m_data_ready);
             serializer(m_x);
             serializer(m_y);
-            serializer(m_perf_length);
             serializer(m_icd);
         }
 
@@ -180,13 +177,6 @@ namespace Opm {
         // the segments whose outlet segments are the current segment
         std::vector<int> m_inlet_segments;
 
-        // length of the segment node to the bhp reference point.
-        // when reading in from deck, with 'INC',
-        // it will be incremental length before processing.
-        // After processing and in the class Well, it always stores the 'ABS' value.
-        // which means the total_length
-        double m_total_length;
-
         // depth of the nodes to the bhp reference point
         // when reading in from deck, with 'INC',
         // it will be the incremental depth before processing.
@@ -194,6 +184,13 @@ namespace Opm {
         // TODO: to check if it is good to use 'ABS' always.
         double m_depth;
 
+        // length of the segment node to the bhp reference point.
+        // when reading in from deck, with 'INC',
+        // it will be incremental length before processing.
+        // After processing and in the class Well, it always stores the 'ABS' value.
+        // which means the total_length
+        double m_total_length;
+
         // tubing internal diameter
         // or the equivalent diameter for annular cross-sections
         // for top segment, it is UNDEFINED
@@ -228,7 +225,6 @@ namespace Opm {
         // simulations, but needed for the SEG option in WRFTPLT.
         double m_y{};
 
-        std::optional<double> m_perf_length;
         std::variant<RegularSegment, SICD, AutoICD, Valve> m_icd;
 
         // There are three other properties for the segment pertaining to
