@@ -74,10 +74,10 @@ namespace Opm {
     class TracerConfig;
     class UDQConfig;
     class Well;
-    enum class WellGasInflowEquation;
+    enum class WellGasInflowEquation : std::uint8_t;
     class WellMatcher;
-    enum class WellProducerCMode;
-    enum class WellStatus;
+    enum class WellProducerCMode : std::uint16_t;
+    enum class WellStatus : std::uint8_t;
     class WelSegsSet;
     class WellTestConfig;
 } // namespace Opm
@@ -106,7 +106,9 @@ namespace Opm {
 
         /*! \brief Construct a Schedule object from a deck.
          *  \param deck Deck to construct Schedule from
+         *  \param grid Eclipse grid description
          *  \param fp Field property manager
+         *  \param numAquifers Numerical aquifers to use
          *  \param runspec Run specification parameters to use
          *  \param parseContext Parsing context
          *  \param errors Error configuration
@@ -122,7 +124,7 @@ namespace Opm {
                  const EclipseGrid& grid,
                  const FieldPropsManager& fp,
                  const NumericalAquifers& numAquifers,
-                 const Runspec &runspec,
+                 const Runspec& runspec,
                  const ParseContext& parseContext,
                  ErrorGuard& errors,
                  std::shared_ptr<const Python> python,
@@ -325,7 +327,6 @@ namespace Opm {
         WellProducerCMode getGlobalWhistctlMmode(std::size_t timestep) const;
 
         const UDQConfig& getUDQConfig(std::size_t timeStep) const;
-        void evalAction(const SummaryState& summary_state, std::size_t timeStep);
 
         GTNode groupTree(std::size_t report_step) const;
         GTNode groupTree(const std::string& root_node, std::size_t report_step) const;
@@ -350,12 +351,14 @@ namespace Opm {
         SimulatorUpdate applyAction(std::size_t reportStep,
                                     const Action::ActionX& action,
                                     const Action::Result::MatchingEntities& matches,
-                                    const std::unordered_map<std::string, double>& wellpi);
+                                    const std::unordered_map<std::string, double>& wellpi,
+                                    const bool iterateSchedule);
 
         SimulatorUpdate applyAction(std::size_t reportStep,
                                     const Action::ActionX& action,
                                     const Action::Result::MatchingEntities& matches,
-                                    const std::unordered_map<std::string, float>& wellpi);
+                                    const std::unordered_map<std::string, float>& wellpi,
+                                    const bool iterateSchedule);
         /*
           The runPyAction() will run the Python script in a PYACTION keyword. In
           the case of Schedule updates the recommended way of doing that from
@@ -391,6 +394,13 @@ namespace Opm {
         bool operator==(const Schedule& data) const;
         std::shared_ptr<const Python> python() const;
 
+        /// Retrieve initial report configuration object
+        ///
+        /// Populated by RPTSOL keyword in the SOLUTION section.
+        ///
+        /// \return Initial configuration object.  Nullopt if there is no
+        /// RPTSOL information in the SOLUTION section.
+        const std::optional<RPTConfig>& initialReportConfiguration() const;
 
         const ScheduleState& back() const;
         const ScheduleState& operator[](std::size_t index) const;
@@ -493,7 +503,7 @@ namespace Opm {
 
         // The current_report_step is set to the current report step when a PYACTION call is executed.
         // This is needed since the Schedule object does not know the current report step of the simulator and
-        // we only allow PYACTIONS for the current and future report steps. 
+        // we only allow PYACTIONS for the current and future report steps.
         std::size_t current_report_step = 0;
         // The simUpdateFromPython points to a SimulatorUpdate collecting all updates from one PYACTION call.
         // The SimulatorUpdate is reset before a new PYACTION call is executed.
@@ -526,7 +536,6 @@ namespace Opm {
 
         void updateGuideRateModel(const GuideRateModel& new_model, std::size_t report_step);
         GTNode groupTree(const std::string& root_node, std::size_t report_step, std::size_t level, const std::optional<std::string>& parent_name) const;
-        bool checkGroups(const ParseContext& parseContext, ErrorGuard& errors);
         bool updateWellStatus( const std::string& well, std::size_t reportStep, WellStatus status, std::optional<KeywordLocation> = {});
         void addWellToGroup( const std::string& group_name, const std::string& well_name , std::size_t timeStep);
         void iterateScheduleSection(std::size_t load_start,
@@ -545,7 +554,7 @@ namespace Opm {
         void addGroup(const RestartIO::RstGroup& rst_group, std::size_t timeStep);
         void addWell(const std::string& wellName, const DeckRecord& record,
                     std::size_t timeStep, ConnectionOrder connection_order);
-        void checkIfAllConnectionsIsShut(std::size_t currentStep);
+        void checkIfAllConnectionsIsShut(std::size_t reportStep);
         void end_report(std::size_t report_step);
         /// \param welsegs_wells All wells with a WELSEGS entry for checks.
         /// \param compegs_wells All wells with a COMPSEGS entry for checks.
@@ -570,9 +579,7 @@ namespace Opm {
         std::vector<std::string> wellNames(const std::string& pattern,
                                            const HandlerContext& context,
                                            bool allowEmpty = false);
-        std::vector<std::string> wellNames(const std::string& pattern, std::size_t timeStep, const std::vector<std::string>& matching_wells, InputErrorAction error_action, ErrorGuard& errors, const KeywordLocation& location) const;
         static std::string formatDate(std::time_t t);
-        std::string simulationDays(std::size_t currentStep) const;
         void applyGlobalWPIMULT( const std::unordered_map<std::string, double>& wpimult_global_factor);
 
         bool must_write_rst_file(std::size_t report_step) const;
