@@ -22,21 +22,25 @@
 
 #include <opm/input/eclipse/Schedule/UDQ/UDQSet.hpp>
 
+#include <opm/output/eclipse/WindowedArray.hpp>
+
 #include <cstddef>
 #include <string>
 #include <unordered_map>
 #include <utility>
-#include <vector>
 
-namespace Opm { namespace RestartIO {
+namespace Opm::RestartIO {
     struct RstState;
-}} // namespace Opm::RestartIO
+    class RstUDQ;
+} // namespace Opm::RestartIO
 
 namespace Opm {
 
 class UDQState
 {
 public:
+    using ExportRange = RestartIO::Helpers::WindowedArray<double>::WriteWindow;
+
     UDQState() = default;
     explicit UDQState(double undefined);
 
@@ -52,9 +56,13 @@ public:
     double get_well_var(const std::string& well, const std::string& var) const;
     double get_segment_var(const std::string& well, const std::string& var, const std::size_t segment) const;
 
+    void exportSegmentUDQ(const std::string& var,
+                          const std::string& well,
+                          ExportRange&       output) const;
+
     void add_define(std::size_t report_step, const std::string& udq_key, const UDQSet& result);
     void add_assign(const std::string& udq_key, const UDQSet& result);
-    bool define(const std::string& udq_key, const std::pair<UDQUpdate, std::size_t>& update_status) const;
+    bool define(const std::pair<UDQUpdate, std::size_t>& update_status) const;
     double undefined_value() const;
 
     bool operator==(const UDQState& other) const;
@@ -73,7 +81,7 @@ public:
     }
 
 private:
-    double undef_value;
+    double undef_value{};
     std::unordered_map<std::string, double> scalar_values{};
 
     // [var][well] -> double
@@ -85,7 +93,7 @@ private:
     // [var][well][segment] -> double
     std::unordered_map<std::string, std::unordered_map<std::string, std::unordered_map<std::size_t, double>>> segment_values{};
 
-    std::unordered_map<std::string, std::size_t> defines;
+    std::unordered_map<std::string, std::size_t> defines{};
 
     void add(const std::string& udq_key, const UDQSet& result);
     double get_wg_var(const std::string& well, const std::string& key, UDQVarType var_type) const;
