@@ -306,7 +306,7 @@ Well::Well(const RestartIO::RstWell& rst_well,
     use_efficiency_in_network(true),   // @TODO@ Find and read the actual value from restart
     solvent_fraction(def_solvent_fraction),
     has_produced(rst_well.void_total != 0),
-    has_injected(rst_well.void_inj_total != 0),    
+    has_injected(rst_well.void_inj_total != 0),
     prediction_mode(rst_well.hist_requested_control == 0),
     econ_limits(economicLimits(rst_well)),
     foam_properties(std::make_shared<WellFoamProperties>()),
@@ -469,9 +469,11 @@ Well::Well(const RestartIO::RstWell& rst_well,
         if (!rst_well.tracer_concentration_injection.empty()) {
             auto tracer = std::make_shared<WellTracerProperties>(this->getTracerProperties());
             for (std::size_t tracer_index = 0; tracer_index < tracer_config.size(); tracer_index++) {
-                const auto& tname = tracer_config[tracer_index].name;
-                const auto concentration = rst_well.tracer_concentration_injection[tracer_index];
-                tracer->setConcentration(tname, concentration);
+                const auto& trName = tracer_config[tracer_index].name;
+                const auto trConcentration = rst_well.tracer_concentration_injection[tracer_index];
+                // currently there is no support to UDA tracer concentrations from restart files
+                tracer->setConcentration(WellTracerProperties::Tracer { trName },
+                                         UDAValue { trConcentration });
             }
             this->updateTracer(tracer);
         }
@@ -550,6 +552,8 @@ Well Well::serializationTestObject()
     result.group_name = "test2";
     result.init_step = 1;
     result.insert_index = 2;
+    result.insert_index_lgr = 30;
+    result.insert_index_all_lgr = 33;
     result.headI = 3;
     result.headJ = 4;
     result.ref_depth = 5;
@@ -605,21 +609,28 @@ void Well::flag_lgr_well(void)
     ref_type = WellRefinementType::LGR;
 }
 
+void Well::setInsertIndexLGR(const std::size_t index)
+{
+    this->insert_index_lgr = index;
+}
+
+void Well::setInsertIndexAllLGR(const std::size_t index)
+{
+    this->insert_index_all_lgr = index;
+}
+
 void Well::set_lgr_well_tag(const std::string& lgr_tag_name)
 {
     lgr_tag = lgr_tag_name;
 }
 
 std::optional<std::string> Well::get_lgr_well_tag(void) const
-{   
+{
     if (this->ref_type == WellRefinementType::STANDARD)
         return std::nullopt;
     return lgr_tag;
 }
 
-
-
-
 bool Well::is_lgr_well(void) const
 {
     return this->ref_type == WellRefinementType::LGR;
