@@ -60,7 +60,8 @@ namespace Opm {
           injectionControls(0),
           injectorType(InjectorType::WATER),
           controlMode(InjectorCMode::CMODE_UNDEFINED),
-          rsRvInj(0.0)
+          rsRvInj(0.0),
+          gas_inj_composition(std::nullopt)
     {
     }
 
@@ -82,6 +83,7 @@ namespace Opm {
         result.injectorType = InjectorType::OIL;
         result.controlMode = InjectorCMode::BHP;
         result.rsRvInj = 11;
+        result.gas_inj_composition = std::vector<double>{1.0, 2.0, 3.0};
 
         return result;
     }
@@ -170,19 +172,19 @@ namespace Opm {
             if (this->injectorType == InjectorType::OIL)
                 this->surfaceInjectionRate.update_value( new_arg );
             else
-                std::invalid_argument("Well type must be OIL to set the oil rate");
+                throw std::invalid_argument("Well type must be OIL to set the oil rate");
         }
         else if (cmode == WELTARGCMode::WRAT){
             if (this->injectorType == InjectorType::WATER)
                 this->surfaceInjectionRate.update_value( new_arg );
             else
-                std::invalid_argument("Well type must be WATER to set the water rate");
+                throw std::invalid_argument("Well type must be WATER to set the water rate");
         }
         else if (cmode == WELTARGCMode::GRAT){
             if(this->injectorType == InjectorType::GAS)
                 this->surfaceInjectionRate.update_value( new_arg );
             else
-                std::invalid_argument("Well type must be GAS to set the gas rate");
+                throw std::invalid_argument("Well type must be GAS to set the gas rate");
         }
         else if (cmode == WELTARGCMode::THP)
             this->THPTarget.update_value( new_arg );
@@ -197,6 +199,7 @@ namespace Opm {
 
     void
     Well::WellInjectionProperties::handleWCONINJH(const DeckRecord& record,
+                                                  const int vfp_table_nr,
                                                   const double bhp_def,
                                                   const bool is_producer,
                                                   const std::string& well_name,
@@ -223,6 +226,7 @@ namespace Opm {
         InjectorCMode newControlMode = WellInjectorCModeFromString(cmodeString);
 
         if ( !(newControlMode == InjectorCMode::RATE || newControlMode == InjectorCMode::BHP) ) {
+            const auto inputControlMode = newControlMode;
             newControlMode = InjectorCMode::RATE;
             const auto& sir = this->surfaceInjectionRate;
             std::string target = sir.is<double>() ? std::to_string(sir.get<double>()) : sir.get<std::string>();
@@ -233,7 +237,7 @@ namespace Opm {
                                           loc.filename,
                                           loc.lineno,
                                           well_name,
-                                          WellInjectorCMode2String(newControlMode),
+                                          WellInjectorCMode2String(inputControlMode),
                                           target);
             OpmLog::warning(msg);
         }
@@ -258,10 +262,8 @@ namespace Opm {
         this->controlMode = newControlMode;
         this->predictionMode = false;
 
-        const int VFPTableNumberArg = record.getItem("VFP_TABLE").get< int >(0);
-        if (VFPTableNumberArg > 0) {
-            this->VFPTableNumber = VFPTableNumberArg;
-        }
+        this->VFPTableNumber = vfp_table_nr;
+
         this->rsRvInj = record.getItem("VAPOIL_C").getSIDouble(0);
         if (this->injectorType == InjectorType::OIL && this->rsRvInj > 0) {
             double factor = record.getItem("VAPOIL_C").getSIDouble(0) / record.getItem("VAPOIL_C").get<double>(0);
@@ -284,7 +286,8 @@ namespace Opm {
             (injectionControls == other.injectionControls) &&
             (injectorType == other.injectorType) &&
             (controlMode == other.controlMode) &&
-            (rsRvInj == other.rsRvInj))
+            (rsRvInj == other.rsRvInj) &&
+            (gas_inj_composition == other.gas_inj_composition))
             return true;
         else
             return false;
@@ -342,7 +345,7 @@ namespace Opm {
         controls.injector_type = this->injectorType;
         controls.cmode = this->controlMode;
         controls.vfp_table_number = this->VFPTableNumber;
-        controls.injector_type = this->injectorType;
+        controls.prediction_mode = this->predictionMode;
         controls.rs_rv_inj = this->rsRvInj;
 
         return controls;
@@ -458,38 +461,57 @@ namespace Opm {
     }
 
     void Well::WellInjectionProperties::handleWTMULT(Well::WELTARGCMode cmode, double factor) {
-        if (cmode == Well::WELTARGCMode::BHP)
-            this->BHPTarget *= factor;
 
-        else if (cmode == WELTARGCMode::ORAT) {
-            if (this->injectorType == InjectorType::OIL)
-                this->surfaceInjectionRate *= factor;
-            else
-                std::invalid_argument("Well type must be OIL to scale the oil rate");
+        auto update_target = [cmode, factor](UDAValue& target) {
+            if (target.is_defined()) {
+                target *= factor;
+                return;
+            }
+            throw std::invalid_argument(fmt::format("Cannot apply WTMULT to undefined {} target", WellWELTARGCMode2String(cmode)));
+        };
+
+        switch (cmode) {
+            case Well::WELTARGCMode::BHP:
+                update_target(this->BHPTarget);
+                break;
+            case Well::WELTARGCMode::ORAT:
+                if (this->injectorType == InjectorType::OIL)
+                    update_target(this->surfaceInjectionRate);
+                else
+                    throw std::invalid_argument("Well type must be OIL to scale the oil rate");
+                break;
+            case Well::WELTARGCMode::WRAT:
+                if (this->injectorType == InjectorType::WATER)
+                    update_target(this->surfaceInjectionRate);
+                else
+                    throw std::invalid_argument("Well type must be WATER to scale the oil rate");
+                break;
+            case Well::WELTARGCMode::GRAT:
+                if (this->injectorType == InjectorType::GAS)
+                    update_target(this->surfaceInjectionRate);
+                else
+                    throw std::invalid_argument("Well type must be GAS to scale the oil rate");
+                break;
+            case Well::WELTARGCMode::THP:
+                update_target(this->THPTarget);
+                break;
+            case Well::WELTARGCMode::RESV:
+                update_target(this->reservoirInjectionRate);
+                break;
+            default:
+                std::invalid_argument(fmt::format("Invalid target {} supplied to WTMULT", WellWELTARGCMode2String(cmode)));
         }
-
-        else if (cmode == WELTARGCMode::WRAT) {
-            if (this->injectorType == InjectorType::WATER)
-                this->surfaceInjectionRate *= factor;
-            else
-                std::invalid_argument("Well type must be WATER to scale the water rate");
-        }
-
-        else if (cmode == WELTARGCMode::GRAT) {
-            if(this->injectorType == InjectorType::GAS)
-                this->surfaceInjectionRate *= factor;
-            else
-                std::invalid_argument("Well type must be GAS to scale the gas rate");
-        }
-
-        else if (cmode == WELTARGCMode::THP)
-            this->THPTarget *= factor;
-
-        else if (cmode == WELTARGCMode::RESV)
-            this->reservoirInjectionRate*= factor;
-
-        else throw std::invalid_argument("Invalid keyword (MODE) supplied");
     }
 
+    void Well::WellInjectionProperties::setGasInjComposition(const std::vector<double>& composition) {
+        gas_inj_composition = composition;
+    }
+
+    const std::vector<double>& Well::WellInjectionProperties::gasInjComposition() const {
+        if (!gas_inj_composition.has_value()) {
+            throw std::invalid_argument("Gas injection composition not set");
+        }
+        return gas_inj_composition.value();
+    }
 
 }
