@@ -18,33 +18,38 @@
  */
 
 #include "WellKeywordHandlers.hpp"
+
 #include <opm/common/OpmLog/OpmLog.hpp>
 #include <opm/common/utility/OpmInputError.hpp>
 #include <opm/common/utility/String.hpp>
 
-#include <opm/input/eclipse/Deck/DeckKeyword.hpp>
-
-#include <opm/input/eclipse/Parser/ParseContext.hpp>
-#include <opm/input/eclipse/Parser/ParserKeywords/F.hpp>
-#include <opm/input/eclipse/Parser/ParserKeywords/W.hpp>
+#include <opm/input/eclipse/EclipseState/Grid/EclipseGrid.hpp>
 
 #include <opm/input/eclipse/Schedule/Network/ExtNetwork.hpp>
+#include <opm/input/eclipse/Schedule/ScheduleGrid.hpp>
 #include <opm/input/eclipse/Schedule/ScheduleState.hpp>
 #include <opm/input/eclipse/Schedule/ScheduleStatic.hpp>
 #include <opm/input/eclipse/Schedule/UDQ/UDQActive.hpp>
 #include <opm/input/eclipse/Schedule/UDQ/UDQConfig.hpp>
 #include <opm/input/eclipse/Schedule/Well/NameOrder.hpp>
 #include <opm/input/eclipse/Schedule/Well/WDFAC.hpp>
-#include <opm/input/eclipse/Schedule/Well/Well.hpp>
-#include <opm/input/eclipse/Schedule/Well/WellConnections.hpp>
-#include <opm/input/eclipse/Schedule/Well/WellEconProductionLimits.hpp>
-#include <opm/input/eclipse/Schedule/Well/WellTestConfig.hpp>
 #include <opm/input/eclipse/Schedule/Well/WListManager.hpp>
 #include <opm/input/eclipse/Schedule/Well/WVFPDP.hpp>
 #include <opm/input/eclipse/Schedule/Well/WVFPEXP.hpp>
+#include <opm/input/eclipse/Schedule/Well/Well.hpp>
+#include <opm/input/eclipse/Schedule/Well/WellConnections.hpp>
+#include <opm/input/eclipse/Schedule/Well/WellEconProductionLimits.hpp>
+#include <opm/input/eclipse/Schedule/Well/WellFractureSeeds.hpp>
+#include <opm/input/eclipse/Schedule/Well/WellTestConfig.hpp>
 
 #include <opm/input/eclipse/Units/UnitSystem.hpp>
 
+#include <opm/input/eclipse/Deck/DeckKeyword.hpp>
+
+#include <opm/input/eclipse/Parser/ParseContext.hpp>
+#include <opm/input/eclipse/Parser/ParserKeywords/F.hpp>
+#include <opm/input/eclipse/Parser/ParserKeywords/W.hpp>
+
 #include "../HandlerContext.hpp"
 
 #include <fmt/format.h>
@@ -52,6 +57,11 @@
 
 #include <algorithm>
 #include <memory>
+#include <numeric>
+#include <string>
+#include <unordered_map>
+#include <unordered_set>
+#include <vector>
 
 namespace Opm {
 
@@ -85,6 +95,17 @@ std::string trim_wgname(const DeckKeyword& keyword,
     return wgname;
 }
 
+void updateOpenShutEvents(HandlerContext& handlerContext, const std::string& well_name){
+    if (handlerContext.getWellStatus(well_name) == WellStatus::OPEN) {
+        handlerContext.state().wellgroup_events().addEvent( well_name, ScheduleEvents::REQUEST_OPEN_WELL);
+        handlerContext.state().wellgroup_events().clearEvent( well_name, ScheduleEvents::REQUEST_SHUT_WELL);
+    }
+    if (handlerContext.getWellStatus(well_name) == WellStatus::SHUT) {
+        handlerContext.state().wellgroup_events().addEvent( well_name, ScheduleEvents::REQUEST_SHUT_WELL);
+        handlerContext.state().wellgroup_events().clearEvent( well_name, ScheduleEvents::REQUEST_OPEN_WELL);
+    }
+}
+
 void handleWCONHIST(HandlerContext& handlerContext)
 {
     for (const auto& record : handlerContext.keyword) {
@@ -167,11 +188,8 @@ void handleWCONHIST(HandlerContext& handlerContext)
                 handlerContext.affected_well(well_name);
             }
 
-            // Always check if well can be opened (it could have been closed for numerical reasons and possible to operate with new params)
-            if (handlerContext.getWellStatus(well_name) == WellStatus::OPEN) {
-                handlerContext.state().wellgroup_events().addEvent( well_name, ScheduleEvents::REQUEST_OPEN_WELL);
-            }
-
+            // Add Event if well open/shut is requested
+            updateOpenShutEvents(handlerContext, well_name);
         }
     }
 }
@@ -245,11 +263,8 @@ void handleWCONINJE(HandlerContext& handlerContext)
                 handlerContext.state().wells.update( std::move(well2) );
                 handlerContext.affected_well(well_name);
             }
-
-            if (handlerContext.state().wells.get( well_name ).getStatus() == Well::Status::OPEN) {
-                handlerContext.state().wellgroup_events().addEvent(well_name, ScheduleEvents::REQUEST_OPEN_WELL);
-            }
-
+             // Add Event if well open/shut is requested
+            updateOpenShutEvents(handlerContext, well_name);
             auto udq_active = handlerContext.state().udq_active.get();
             if (injection->updateUDQActive(handlerContext.state().udq.get(), udq_active)) {
                 handlerContext.state().udq_active.update( std::move(udq_active) );
@@ -324,10 +339,8 @@ void handleWCONINJH(HandlerContext& handlerContext)
                 handlerContext.affected_well(well_name);
             }
 
-            // Always check if well can be opened (it could have been closed for numerical reasons and possible to operate with new params)
-            if (handlerContext.getWellStatus(well_name) == WellStatus::OPEN) {
-                handlerContext.state().wellgroup_events().addEvent( well_name, ScheduleEvents::REQUEST_OPEN_WELL);
-            }
+            // Add Event if well open/shut is requested
+            updateOpenShutEvents(handlerContext, well_name);
         }
     }
 }
@@ -416,10 +429,6 @@ void handleWCONPROD(HandlerContext& handlerContext)
                 update_well = true;
             }
 
-            if (well2.getStatus() == WellStatus::OPEN) {
-                handlerContext.state().wellgroup_events().addEvent(well2.name(), ScheduleEvents::REQUEST_OPEN_WELL);
-            }
-
             if (update_well) {
                 handlerContext.state().events().addEvent( ScheduleEvents::PRODUCTION_UPDATE );
                 handlerContext.state().wellgroup_events().addEvent( well2.name(), ScheduleEvents::PRODUCTION_UPDATE);
@@ -427,6 +436,9 @@ void handleWCONPROD(HandlerContext& handlerContext)
                 handlerContext.affected_well(well_name);
             }
 
+            // Add Event if well open/shut is requested
+            updateOpenShutEvents(handlerContext, well_name);
+
             auto udq_active = handlerContext.state().udq_active.get();
             if (properties->updateUDQActive(handlerContext.state().udq.get(), udq_active)) {
                 handlerContext.state().udq_active.update( std::move(udq_active));
@@ -514,10 +526,8 @@ void handleWELOPEN(HandlerContext& handlerContext)
                         handlerContext.state().wells.update(std::move(well2));
                     }
                 }
-
-                if (new_well_status == open) {
-                    handlerContext.state().wellgroup_events().addEvent( wname, ScheduleEvents::REQUEST_OPEN_WELL);
-                }
+                // Add Event if well open/shut is requested
+                updateOpenShutEvents(handlerContext, wname);
             }
             continue;
         }
@@ -529,11 +539,6 @@ void handleWELOPEN(HandlerContext& handlerContext)
           shut.
          */
         for (const auto& wname : well_names) {
-            {
-                auto well = handlerContext.state().wells.get(wname);
-                handlerContext.state().wells.update( std::move(well) );
-            }
-
             const auto connection_status = Connection::StateFromString( status_str );
             {
                 auto well = handlerContext.state().wells.get(wname);
@@ -698,6 +703,7 @@ Well{0} entered with 'FIELD' parent group:
 void handleWELSPECL(HandlerContext& handlerContext)
 {
     using Kw = ParserKeywords::WELSPECL;
+
     auto getTrimmedName = [&handlerContext](const auto& item)
     {
         return trim_wgname(handlerContext.keyword,
@@ -706,43 +712,75 @@ void handleWELSPECL(HandlerContext& handlerContext)
                            handlerContext.errors);
     };
     handleWELSPECS(handlerContext);
+    std::size_t index = 0;
+    std::unordered_map<std::string, int> lgr_well_seq_map;
+
     for (const auto& record : handlerContext.keyword) {
         const auto wellName = getTrimmedName(record.getItem<Kw::WELL>());
         const auto lgrTag = getTrimmedName(record.getItem<Kw::LGR>());
-        handlerContext.state().wells.get(wellName).flag_lgr_well();
-        handlerContext.state().wells.get(wellName).set_lgr_well_tag(lgrTag);
+        const auto& [tagPos, inserted] = lgr_well_seq_map.try_emplace(lgrTag, 0);
+        if (! inserted) {
+            // lgrTag already exists in the map, increase sequence number.
+            ++tagPos->second;
+        }
+        auto& well = handlerContext.state().wells.get(wellName);
+        well.setInsertIndexLGR(tagPos->second);
+        well.setInsertIndexAllLGR(index);
+        well.flag_lgr_well();
+        well.set_lgr_well_tag(lgrTag);
+        index++;
     }
 }
 
-/*
-  The documentation for the WELTARG keyword says that the well
-  must have been fully specified and initialized using one of the
-  WCONxxxx keywords prior to modifying the well using the WELTARG
-  keyword.
-
-  The following implementation of handling the WELTARG keyword
-  does not check or enforce in any way that this is done (i.e. it
-  is not checked or verified that the well is initialized with any
-  WCONxxxx keyword).
-
-  Update: See the discussion following the definitions of the SI factors, due
-  to a bad design we currently need the well to be specified with
-  WCONPROD / WCONHIST before WELTARG is applied, if not the units for the
-  rates will be wrong.
-*/
+// The documentation for the WELTARG keyword says that the well must have
+// been fully specified and initialized using one of the WCONxxxx keywords
+// prior to modifying the well using the WELTARG keyword.
+//
+// The following implementation of handling the WELTARG keyword does not
+// check or enforce in any way that this is done (i.e., it is not checked or
+// verified that the well is initialized with any WCONxxxx keyword).
+//
+// Update: See the discussion following the definitions of the SI factors,
+// due to a bad design we currently need the well to be specified with
+// WCONPROD / WCONHIST before WELTARG is applied.  Otherwise the units for
+// the rates will be wrong.
 
 void handleWELTARG(HandlerContext& handlerContext)
 {
-    const double SiFactorP = handlerContext.static_schedule().m_unit_system.parse("Pressure").getSIScaling();
+    using Kw = ParserKeywords::WELTARG;
+
+    const auto SiFactorP = handlerContext.static_schedule()
+        .m_unit_system.parse("Pressure").getSIScaling();
+
     for (const auto& record : handlerContext.keyword) {
-        const std::string& wellNamePattern = record.getItem("WELL").getTrimmedString(0);
-        const auto well_names = handlerContext.wellNames(wellNamePattern);
+        const auto wellNamePattern = record.getItem<Kw::WELL>().getTrimmedString(0);
+
+        const auto is_wlist = handlerContext.state()
+            .wlist_manager().hasList(wellNamePattern);
+
+        const auto well_names = handlerContext.wellNames(wellNamePattern, is_wlist);
+
         if (well_names.empty()) {
-            handlerContext.invalidNamePattern( wellNamePattern);
+            if (is_wlist) {
+                // wellNamePattern names an empty well list.  This is okay,
+                // so issue a warning and continue.
+                const auto msg_format =
+                    fmt::format("Empty WLIST '{}' in '{{keyword}}', "
+                                "in {{file}} line {{line}}.", wellNamePattern);
+
+                const auto msg = OpmInputError::format
+                    (msg_format, handlerContext.keyword.location());
+
+                OpmLog::warning("WELTARG:EmptyWLIST", msg);
+                continue;
+            }
+            else {
+                handlerContext.invalidNamePattern(wellNamePattern);
+            }
         }
 
-        const auto cmode = WellWELTARGCModeFromString(record.getItem("CMODE").getTrimmedString(0));
-        const auto new_arg = record.getItem("NEW_VALUE").get<UDAValue>(0);
+        const auto cmode = WellWELTARGCModeFromString(record.getItem<Kw::CMODE>().getTrimmedString(0));
+        const auto new_arg = record.getItem<Kw::NEW_VALUE>().get<UDAValue>(0);
 
         for (const auto& well_name : well_names) {
             auto well2 = handlerContext.state().wells.get(well_name);
@@ -849,24 +887,38 @@ void handleWHISTCTL(HandlerContext& handlerContext)
 
 void handleWLIST(HandlerContext& handlerContext)
 {
-    const std::string legal_actions = "NEW:ADD:DEL:MOV";
+    enum class WellListAction { New, Add, Del, Mov, Invalid };
+
+    auto parseAction = [](const std::string& action) {
+        if (action == "NEW") return WellListAction::New;
+        if (action == "ADD") return WellListAction::Add;
+        if (action == "DEL") return WellListAction::Del;
+        if (action == "MOV") return WellListAction::Mov;
+        return WellListAction::Invalid;
+    };
+
     for (const auto& record : handlerContext.keyword) {
-        const std::string& name = record.getItem("NAME").getTrimmedString(0);
-        const std::string& action = record.getItem("ACTION").getTrimmedString(0);
+        const std::string name = record.getItem("NAME").getTrimmedString(0);
+        const std::string action_str = record.getItem("ACTION").getTrimmedString(0);
         const std::vector<std::string>& well_args = record.getItem("WELLS").getData<std::string>();
         std::vector<std::string> wells;
         auto new_wlm = handlerContext.state().wlist_manager.get();
 
-        if (legal_actions.find(action) == std::string::npos)
-            throw std::invalid_argument("The action:" + action + " is not recognized.");
+        WellListAction action = parseAction(action_str);
+        if (action == WellListAction::Invalid) {
+                const auto& parseContext = handlerContext.parseContext;
+                parseContext.handleError(ParseContext::SCHEDULE_INVALID_NAME,
+                fmt::format("The action: {} is not recognized.", action_str),
+                handlerContext.keyword.location(), handlerContext.errors);
+        }
 
         for (const auto& well_arg : well_args) {
             // does not use overload for context to avoid throw
             const auto names = handlerContext.wellNames(well_arg, true);
             if (names.empty() && well_arg.find("*") == std::string::npos) {
-                std::string msg_fmt = "Problem with {keyword}\n"
-                                      "In {file} line {line}\n"
-                                      "The well '" + well_arg + "' has not been defined with WELSPECS and will not be added to the list.";
+                const std::string msg_fmt = "Problem with {keyword}\n"
+                                            "In {file} line {line}\n"
+                                            "The well '" + well_arg + "' has not been defined with WELSPECS and will not be added to the list.";
                 const auto& parseContext = handlerContext.parseContext;
                 parseContext.handleError(ParseContext::SCHEDULE_INVALID_NAME, msg_fmt, handlerContext.keyword.location(),handlerContext.errors);
                 continue;
@@ -878,27 +930,41 @@ void handleWLIST(HandlerContext& handlerContext)
         if (name[0] != '*')
             throw std::invalid_argument("The list name in WLIST must start with a '*'");
 
-        if (action == "NEW") {
+        switch (action) {
+        case WellListAction::New: {
             new_wlm.newList(name, wells);
+            break;
         }
-
-        if (!new_wlm.hasList(name))
-            throw std::invalid_argument("Invalid well list: " + name);
-
-        if (action == "MOV") {
+        case WellListAction::Add: {
+            new_wlm.addOrCreateWellList(name, wells);
+            break;
+        }
+        case WellListAction::Mov: {
             for (const auto& well : wells) {
                 new_wlm.delWell(well);
             }
+            new_wlm.addOrCreateWellList(name, wells);
+            break;
         }
-
-        if (action == "DEL") {
-            for (const auto& well : wells) {
-                new_wlm.delWListWell(well, name);
-            }
-        } else if (action != "NEW"){
-            for (const auto& well : wells) {
-                new_wlm.addWListWell(well, name);
+        case WellListAction::Del: {
+            if (new_wlm.hasList(name)) {
+                for (const auto& well : wells) {
+                    new_wlm.delWListWell(well, name);
+                }
+            } else {
+                const std::string msg_fmt = fmt::format("Problem with {{keyword}} in {{file}} line {{line}}\n"
+                                                        "The well list '{}' has not been defined and the operation DEL can not be applied.",
+                                                         name);
+                const auto& parseContext = handlerContext.parseContext;
+                parseContext.handleError(ParseContext::SCHEDULE_INVALID_NAME,
+                                         msg_fmt,
+                                         handlerContext.keyword.location(),
+                                         handlerContext.errors);
             }
+            break;
+        }
+        default:
+            break;
         }
         handlerContext.state().wlist_manager.update( std::move(new_wlm) );
     }
@@ -967,6 +1033,83 @@ void handleWPAVEDEP(HandlerContext& handlerContext)
     }
 }
 
+/// Handler function for well fracturing seeds
+///
+/// Keyword structure:
+///
+///   WSEED
+///     WellName  I  J  K  nx  ny  nz  ev  eh  wd /
+///     WellName  I  J  K  nx  ny  nz  ev  eh  wd /
+///     WellName  I  J  K  nx  ny  nz  ev  eh  wd /
+///   /
+///
+/// in which 'WellName' is a well, well list, well template or well list
+/// template.  I,J,K are regular well connection coordinates, nx,ny,nz are
+/// the components of the fracturing plane's normal vector, and ev,eh,wd are
+/// the vertical and horizontal extents along with the initial fracture
+/// width at the seed point.
+void handleWSEED(HandlerContext& handlerContext)
+{
+    const auto* grid = handlerContext.grid.get_grid();
+    if (grid == nullptr) {
+        return;
+    }
+
+    auto& seeds = handlerContext.state().wseed;
+    auto updated_seed_wells = std::unordered_set<std::string>{};
+
+    for (const auto& record : handlerContext.keyword) {
+        const auto wellNamePattern = record.getItem<ParserKeywords::WSEED::WELL>()
+            .getTrimmedString(0);
+
+        const auto well_names = handlerContext.wellNames(wellNamePattern, false);
+        if (well_names.empty()) {
+            handlerContext.invalidNamePattern(wellNamePattern);
+            continue;
+        }
+
+        // Subtract one to convert one-based input indices to zero-based
+        // internal processing indices.
+        const auto cellSeedIndex = grid->getGlobalIndex
+            (record.getItem<ParserKeywords::WSEED::I>().get<int>(0) - 1,
+             record.getItem<ParserKeywords::WSEED::J>().get<int>(0) - 1,
+             record.getItem<ParserKeywords::WSEED::K>().get<int>(0) - 1);
+
+        const auto cellSeedNormal = WellFractureSeeds::NormalVector {
+            record.getItem<ParserKeywords::WSEED::NORMAL_X>().getSIDouble(0),
+            record.getItem<ParserKeywords::WSEED::NORMAL_Y>().getSIDouble(0),
+            record.getItem<ParserKeywords::WSEED::NORMAL_Z>().getSIDouble(0),
+        };
+
+        const auto cellSeedSize = WellFractureSeeds::SeedSize {}
+            .verticalExtent  (record.getItem<ParserKeywords::WSEED::SIZE_Z>().getSIDouble(0))
+            .horizontalExtent(record.getItem<ParserKeywords::WSEED::SIZE_H>().getSIDouble(0))
+            .width           (record.getItem<ParserKeywords::WSEED::WIDTH> ().getSIDouble(0));
+
+        for (const auto& well_name : well_names) {
+            const auto hasConn = handlerContext.state()
+                .wells(well_name)
+                .getConnections()
+                .hasGlobalIndex(cellSeedIndex);
+
+            if (! hasConn) { continue; }
+
+            auto seed = seeds.has(well_name)
+                ? std::make_shared<WellFractureSeeds>(seeds(well_name))
+                : std::make_shared<WellFractureSeeds>(well_name);
+
+            if (seed->updateSeed(cellSeedIndex, cellSeedNormal, cellSeedSize)) {
+                updated_seed_wells.insert(well_name);
+                seeds.update(well_name, std::move(seed));
+            }
+        }
+    }
+
+    for (const auto& updated_seed_well : updated_seed_wells) {
+        seeds.get(updated_seed_well).finalizeSeeds();
+    }
+}
+
 void handleWTEST(HandlerContext& handlerContext)
 {
     auto new_config = handlerContext.state().wtest_config.get();
@@ -1014,6 +1157,7 @@ getWellHandlers()
         { "WLIST"   , &handleWLIST    },
         { "WPAVE"   , &handleWPAVE    },
         { "WPAVEDEP", &handleWPAVEDEP },
+        { "WSEED"   , &handleWSEED    },
         { "WTEST"   , &handleWTEST    },
     };
 }
