@@ -24,6 +24,8 @@
 #include <opm/input/eclipse/Schedule/ScheduleTypes.hpp>
 #include <opm/input/eclipse/Schedule/Well/WellEnums.hpp>
 
+#include <cmath>
+
 namespace Opm {
 
 struct WellProductionControls {
@@ -54,6 +56,45 @@ public:
         this->controls = 0;
     }
 
+    bool anyZeroRateConstraint() const {
+        auto is_zero = [](const double x)
+        {
+            return std::isfinite(x) && !std::isnormal(x);
+        };
+
+        if (this->hasControl(WellProducerCMode::ORAT) && is_zero(this->oil_rate)) {
+            return true;
+        }
+
+        if (this->hasControl(WellProducerCMode::WRAT) && is_zero(this->water_rate)) {
+            return true;
+        }
+
+        if (this->hasControl(WellProducerCMode::GRAT) && is_zero(this->gas_rate)) {
+            return true;
+        }
+
+        if (this->hasControl(WellProducerCMode::LRAT) && is_zero(this->liquid_rate)) {
+            return true;
+        }
+
+        if (this->hasControl(WellProducerCMode::RESV)) {
+            // currently, the RESV constraint for a history matching producers will be calculated based on the reservoir condition
+            // at the moment it is used, and we do not have a fixed resv_rate to check against zero. as a result,
+            // we check whether the historic rates for oil, gas and water are all zero to determine whether it is under zero
+            // RESV constraint for history matching producers.
+            const auto zero_rate = this->prediction_mode
+                ? is_zero(this->resv_rate)
+                : is_zero(this->oil_rate) && is_zero(this->water_rate) && is_zero(this->gas_rate);
+
+            if (zero_rate) {
+                return true;
+            }
+        }
+
+        return false;
+    }
+
     bool operator==(const WellProductionControls& other) const
     {
         return this->cmode == other.cmode &&
