@@ -82,8 +82,7 @@ namespace Opm {
             energy_rate,
             icd_strength,
             aicd_strength,
-            polymer_density,
-            salinity,
+            concentration,
             gas_oil_ratio_rate,
             moles,
             ppm,
