@@ -279,10 +279,8 @@ namespace Opm {
         constexpr const double GasDissolutionFactor = GasSurfaceVolume/LiquidSurfaceVolume;
         constexpr const double OilDissolutionFactor = LiquidSurfaceVolume/GasSurfaceVolume;
         constexpr const double Density              = kilogram/cubic(meter);
-        constexpr const double PolymerDensity       = kilogram/cubic(meter);
+        constexpr const double Concentration        = kilogram/cubic(meter);
         constexpr const double FoamDensity          = kilogram/cubic(meter);
-        constexpr const double FoamSurfactantConcentration = kilogram/cubic(meter);
-        constexpr const double Salinity             = kilogram/cubic(meter);
         constexpr const double Viscosity            = centi*Poise;
         constexpr const double Timestep             = day;
         constexpr const double SurfaceTension       = dyne/(centi*meter);
@@ -316,10 +314,8 @@ namespace Opm {
         constexpr const double GasDissolutionFactor = GasSurfaceVolume/LiquidSurfaceVolume;
         constexpr const double OilDissolutionFactor = LiquidSurfaceVolume/GasSurfaceVolume;
         constexpr const double Density              = pound/cubic(feet);
-        constexpr const double PolymerDensity       = pound/stb;
+        constexpr const double Concentration        = pound/stb;
         constexpr const double FoamDensity          = pound/GasSurfaceVolume;
-        constexpr const double FoamSurfactantConcentration = pound/stb;
-        constexpr const double Salinity             = pound/stb;
         constexpr const double Viscosity            = centi*Poise;
         constexpr const double Timestep             = day;
         constexpr const double SurfaceTension       = dyne/(centi*meter);
@@ -353,10 +349,8 @@ namespace Opm {
         constexpr const double GasDissolutionFactor = GasSurfaceVolume/LiquidSurfaceVolume;
         constexpr const double OilDissolutionFactor = LiquidSurfaceVolume/GasSurfaceVolume;
         constexpr const double Density              = gram/cubic(centi*meter);
-        constexpr const double PolymerDensity       = gram/cubic(centi*meter);
+        constexpr const double Concentration        = gram/cubic(centi*meter);
         constexpr const double FoamDensity          = gram/cubic(centi*meter);
-        constexpr const double FoamSurfactantConcentration = gram/cubic(centi*meter);
-        constexpr const double Salinity             = gram/cubic(centi*meter);
         constexpr const double Viscosity            = centi*Poise;
         constexpr const double Timestep             = hour;
         constexpr const double SurfaceTension       = dyne/(centi*meter);
@@ -390,10 +384,8 @@ namespace Opm {
         constexpr const double GasDissolutionFactor = GasSurfaceVolume/LiquidSurfaceVolume;
         constexpr const double OilDissolutionFactor = LiquidSurfaceVolume/GasSurfaceVolume;
         constexpr const double Density              = kilogram/cubic(meter);
-        constexpr const double PolymerDensity       = kilogram/cubic(meter);
+        constexpr const double Concentration        = kilogram/cubic(meter);
         constexpr const double FoamDensity          = kilogram/cubic(meter);
-        constexpr const double FoamSurfactantConcentration = kilogram/cubic(meter);
-        constexpr const double Salinity             = kilogram/cubic(meter);
         constexpr const double Viscosity            = centi*Poise;
         constexpr const double Timestep             = day;
         constexpr const double SurfaceTension       = dyne/(centi*meter);
