@@ -355,6 +355,11 @@ RstState::RstState(std::shared_ptr<EclIO::RestartFileView> rstView,
 void RstState::load_oil_vaporization(const std::vector<int>& intehead,
                                      const std::vector<double>& doubhead)
 {
+    if (!(doubhead[VI::doubhead::dRsDt] < 1.0e+20)) {
+        // DRSDT is not defined in the restart file, so we don't need to update the oilvap object.
+        return;
+    }
+
     const std::size_t numPvtRegions = this->oilvap.numPvtRegions();
     const auto tconv = this->unit_system.to_si(::Opm::UnitSystem::measure::time, 1.0);
     std::vector<double> maximums(numPvtRegions, doubhead[VI::doubhead::dRsDt]/tconv);
@@ -523,18 +528,21 @@ void RstState::add_msw(const std::vector<std::string>& zwel,
 
 void RstState::add_udqs(std::shared_ptr<EclIO::RestartFileView> rstView)
 {
-    const auto& iudq = rstView->getKeyword<int>("IUDQ");
-    const auto& zudn = rstView->getKeyword<std::string>("ZUDN");
-    const auto& zudl = rstView->getKeyword<std::string>("ZUDL");
-
     if (rstView->hasKeyword<int>("IUAD")) {
+        const auto rstFileVersion = rstView->
+            getKeyword<int>("INTEHEAD")[VI::intehead::VERSION];
+
         const auto& iuad = rstView->getKeyword<int>("IUAD");
         const auto& iuap = rstView->getKeyword<int>("IUAP");
         const auto& igph = rstView->getKeyword<int>("IGPH");
 
-        this->udq_active = RstUDQActive(iuad, iuap, igph);
+        this->udq_active.emplace(rstFileVersion, iuad, iuap, igph);
     }
 
+    const auto& iudq = rstView->getKeyword<int>("IUDQ");
+    const auto& zudn = rstView->getKeyword<std::string>("ZUDN");
+    const auto& zudl = rstView->getKeyword<std::string>("ZUDL");
+
     auto udqValues = UDQVectors { std::move(rstView) };
 
     for (auto udq_index = 0*this->header.num_udq();
