@@ -257,11 +257,20 @@ RstRecord::RstRecord(const UDAControl  c,
 {}
 
 Opm::RestartIO::RstUDQActive::
-RstUDQActive(const std::vector<int>& iuad_arg,
+RstUDQActive(const int rstFileVersion,
+             const std::vector<int>& iuad_arg,
              const std::vector<int>& iuap,
              const std::vector<int>& igph)
-    : wg_index { iuap }
+    : udaVersion { UDQ::rstFileUDAVersion(rstFileVersion) }
+    , wg_index   { iuap }
 {
+    if (this->udaVersion == UDQ::RstFileUDAVersion::vUnsupp) {
+        throw std::invalid_argument {
+            fmt::format("Restart file version {} is unsupported "
+                        "for UDA restarting.", rstFileVersion)
+        };
+    }
+
     using Ix = Opm::RestartIO::Helpers::VectorItems::IUad::index;
 
     this->iuad.reserve(iuad_arg.size() / UDQDims::entriesPerIUAD());
@@ -272,17 +281,13 @@ RstUDQActive(const std::vector<int>& iuad_arg,
     {
         const auto* uda = &iuad_arg[offset];
 
-        this->iuad.emplace_back(UDQ::udaControl(uda[Ix::UDACode]),
+        this->iuad.emplace_back(UDQ::udaControl(uda[Ix::UDACode], this->udaVersion),
                                 uda[Ix::UDQIndex] - 1,
                                 uda[Ix::NumIuapElm],
                                 uda[Ix::UseCount],
                                 uda[Ix::Offset] - 1);
     }
 
-    std::transform(this->wg_index.begin(), this->wg_index.end(),
-                   this->wg_index.begin(),
-                   [](const int wgIdx) { return wgIdx - 1; });
-
     this->ig_phase.assign(igph.size(), Phase::OIL);
     std::transform(igph.begin(), igph.end(), this->ig_phase.begin(),
                    [](const int phase)
