@@ -45,12 +45,12 @@ struct fmt::formatter<Opm::DenseAd::Evaluation<ValueT,numDerivs,staticSize>>
 
     template<class FormatContext>
     auto format(const Opm::DenseAd::Evaluation<ValueT,numDerivs,staticSize>& e,
-                FormatContext& ctx)
+                FormatContext& ctx) const
     {
         std::vector<ValueT> tmp(e.size());
         for (int i = 0; i < e.size(); ++i)
             tmp[i] = e.derivative(i);
-        return fmt::format_to(ctx.out(), "v: "+ spec +" / d: [" + spec +"]",
+        return fmt::format_to(ctx.out(), fmt::runtime("v: "+ spec +" / d: [" + spec +"]"),
                               e.value(), fmt::join(tmp, ", "));
     }
 };
