@@ -133,16 +133,16 @@ public:
      * \param params Parameters
      * \param state The fluid state
      */
-    template <class ContainerT, class FluidState>
+    template <class ContainerT, class FluidState, class ...Args>
     static void capillaryPressures(ContainerT& values,
                                    const Params& params,
                                    const FluidState& state)
     {
-        OPM_TIMEFUNCTION_LOCAL();
+        OPM_TIMEFUNCTION_LOCAL(Subsystem::SatProps);
         using Evaluation = typename std::remove_reference<decltype(values[0])>::type;
-        values[gasPhaseIdx] = pcgn<FluidState, Evaluation>(params, state);
+        values[gasPhaseIdx] = pcgn<FluidState, Evaluation, Args...>(params, state);
         values[oilPhaseIdx] = 0;
-        values[waterPhaseIdx] = - pcnw<FluidState, Evaluation>(params, state);
+        values[waterPhaseIdx] = - pcnw<FluidState, Evaluation, Args...>(params, state);
 
         Valgrind::CheckDefined(values[gasPhaseIdx]);
         Valgrind::CheckDefined(values[oilPhaseIdx]);
@@ -161,12 +161,14 @@ public:
                                          Scalar& swMin,
                                          const Params& params)
     {
-        soMax = 1.0 - params.oilWaterParams().krnSwMdc();
-        swMax = params.oilWaterParams().krwSwMdc();
-        swMin = params.oilWaterParams().pcSwMdc();
-        Valgrind::CheckDefined(soMax);
-        Valgrind::CheckDefined(swMax);
-        Valgrind::CheckDefined(swMin);
+        if constexpr (Traits::enableHysteresis) {
+            soMax = 1.0 - params.oilWaterParams().krnSwMdc();
+            swMax = params.oilWaterParams().krwSwMdc();
+            swMin = params.oilWaterParams().pcSwMdc();
+            Valgrind::CheckDefined(soMax);
+            Valgrind::CheckDefined(swMax);
+            Valgrind::CheckDefined(swMin);
+        }
     }
 
     /*
@@ -181,7 +183,9 @@ public:
                                             const Scalar& swMin,
                                             Params& params)
     {
-        params.oilWaterParams().update(swMin, swMax, 1.0 - soMax);
+        if constexpr (Traits::enableHysteresis) {
+            params.oilWaterParams().update(swMin, swMax, 1.0 - soMax);
+        }
     }
 
     /*
@@ -196,14 +200,15 @@ public:
                                        Scalar& soMin,
                                        const Params& params)
     {
-        const auto Swco = params.Swl();
-        sgMax = 1.0 - params.gasOilParams().krnSwMdc() - Swco;
-        shMax = params.gasOilParams().krwSwMdc();
-        soMin = params.gasOilParams().pcSwMdc();
-
-        Valgrind::CheckDefined(sgMax);
-        Valgrind::CheckDefined(shMax);
-        Valgrind::CheckDefined(soMin);
+        if constexpr (Traits::enableHysteresis) {
+            const auto Swco = params.Swl();
+            sgMax = 1.0 - params.gasOilParams().krnSwMdc() - Swco;
+            shMax = params.gasOilParams().krwSwMdc();
+            soMin = params.gasOilParams().pcSwMdc();
+            Valgrind::CheckDefined(sgMax);
+            Valgrind::CheckDefined(shMax);
+            Valgrind::CheckDefined(soMin);
+        }
     }
 
     /*
@@ -218,8 +223,10 @@ public:
                                           const Scalar& soMin,
                                           Params& params)
     {
-        const auto Swco = params.Swl();
-        params.gasOilParams().update(soMin, shMax, 1.0 - sgMax - Swco);
+        if constexpr (Traits::enableHysteresis) {
+            const auto Swco = params.Swl();
+            params.gasOilParams().update(soMin, shMax, 1.0 - sgMax - Swco);
+        }
     }
 
     static Scalar trappedGasSaturation(const Params& params, bool maximumTrapping)
@@ -253,14 +260,14 @@ public:
      * p_{c,gn} = p_g - p_n
      * \f]
      */
-    template <class FluidState, class Evaluation = typename FluidState::Scalar>
+    template <class FluidState, class Evaluation, class ...Args>
     static Evaluation pcgn(const Params& params,
                            const FluidState& fs)
     {
-        OPM_TIMEFUNCTION_LOCAL();
+        OPM_TIMEFUNCTION_LOCAL(Subsystem::SatProps);
         // Maximum attainable oil saturation is 1-SWL.
         const auto Sw = 1.0 - params.Swl() - decay<Evaluation>(fs.saturation(gasPhaseIdx));
-        return GasOilMaterialLaw::twoPhaseSatPcnw(params.gasOilParams(), Sw);
+        return GasOilMaterialLaw::template twoPhaseSatPcnw<Evaluation, Args...>(params.gasOilParams(), Sw);
     }
 
     /*!
@@ -272,13 +279,13 @@ public:
      * p_{c,nw} = p_n - p_w
      * \f]
      */
-    template <class FluidState, class Evaluation = typename FluidState::Scalar>
+    template <class FluidState, class Evaluation, class ...Args>
     static Evaluation pcnw(const Params& params,
                            const FluidState& fs)
     {
-        OPM_TIMEFUNCTION_LOCAL();
+        OPM_TIMEFUNCTION_LOCAL(Subsystem::SatProps);
         const auto Sw = decay<Evaluation>(fs.saturation(waterPhaseIdx));
-        return OilWaterMaterialLaw::twoPhaseSatPcnw(params.oilWaterParams(), Sw);
+        return OilWaterMaterialLaw::template twoPhaseSatPcnw<Evaluation, Args...>(params.oilWaterParams(), Sw);
     }
 
     /*!
@@ -337,52 +344,52 @@ public:
      * oil relative permeability models" section of the ECLipse
      * technical description.
      */
-    template <class ContainerT, class FluidState>
+    template <class ContainerT, class FluidState, class ...Args>
     static void relativePermeabilities(ContainerT& values,
                                        const Params& params,
                                        const FluidState& fluidState)
     {
-        OPM_TIMEFUNCTION_LOCAL();
+        OPM_TIMEFUNCTION_LOCAL(Subsystem::SatProps);
         using Evaluation = typename std::remove_reference<decltype(values[0])>::type;
 
-        values[waterPhaseIdx] = krw<FluidState, Evaluation>(params, fluidState);
-        values[oilPhaseIdx] = krn<FluidState, Evaluation>(params, fluidState);
-        values[gasPhaseIdx] = krg<FluidState, Evaluation>(params, fluidState);
+        values[waterPhaseIdx] = krw<FluidState, Evaluation, Args...>(params, fluidState);
+        values[oilPhaseIdx] = krn<FluidState, Evaluation, Args...>(params, fluidState);
+        values[gasPhaseIdx] = krg<FluidState, Evaluation, Args...>(params, fluidState);
     }
 
     /*!
      * \brief The relative permeability of the gas phase.
      */
-    template <class FluidState, class Evaluation = typename FluidState::Scalar>
+    template <class FluidState, class Evaluation, class ...Args>
     static Evaluation krg(const Params& params,
                           const FluidState& fluidState)
     {
-        OPM_TIMEFUNCTION_LOCAL();
+        OPM_TIMEFUNCTION_LOCAL(Subsystem::SatProps);
         // Maximum attainable oil saturation is 1-SWL.
         const Evaluation sw = 1.0 - params.Swl() - decay<Evaluation>(fluidState.saturation(gasPhaseIdx));
-        return GasOilMaterialLaw::twoPhaseSatKrn(params.gasOilParams(), sw);
+        return GasOilMaterialLaw::template twoPhaseSatKrn<Evaluation, Args...>(params.gasOilParams(), sw);
     }
 
     /*!
      * \brief The relative permeability of the wetting phase.
      */
-    template <class FluidState, class Evaluation = typename FluidState::Scalar>
+    template <class FluidState, class Evaluation, class ...Args>
     static Evaluation krw(const Params& params,
                           const FluidState& fluidState)
     {
-        OPM_TIMEFUNCTION_LOCAL();
+        OPM_TIMEFUNCTION_LOCAL(Subsystem::SatProps);
         const Evaluation sw = decay<Evaluation>(fluidState.saturation(waterPhaseIdx));
-        return OilWaterMaterialLaw::twoPhaseSatKrw(params.oilWaterParams(), sw);
+        return OilWaterMaterialLaw::template twoPhaseSatKrw<Evaluation, Args...>(params.oilWaterParams(), sw);
     }
 
     /*!
      * \brief The relative permeability of the non-wetting (i.e., oil) phase.
      */
-    template <class FluidState, class Evaluation = typename FluidState::Scalar>
+    template <class FluidState, class Evaluation, class ...Args>
     static Evaluation krn(const Params& params,
                           const FluidState& fluidState)
     {
-        OPM_TIMEFUNCTION_LOCAL();
+        OPM_TIMEFUNCTION_LOCAL(Subsystem::SatProps);
         const Scalar Swco = params.Swl();
 
         const Evaluation sw =
@@ -392,8 +399,8 @@ public:
         const Evaluation sg = decay<Evaluation>(fluidState.saturation(gasPhaseIdx));
 
         const Evaluation Sw_ow = sg + sw;
-        const Evaluation kro_ow = relpermOilInOilWaterSystem<Evaluation>(params, fluidState);
-        const Evaluation kro_go = relpermOilInOilGasSystem<Evaluation>(params, fluidState);
+        const Evaluation kro_ow = relpermOilInOilWaterSystem<Evaluation, FluidState, Args...>(params, fluidState);
+        const Evaluation kro_go = relpermOilInOilGasSystem<Evaluation, FluidState, Args...>(params, fluidState);
 
         // avoid the division by zero: chose a regularized kro which is used if Sw - Swco
         // < epsilon/2 and interpolate between the oridinary and the regularized kro between
@@ -417,11 +424,11 @@ public:
     /*!
      * \brief The relative permeability of oil in oil/gas system.
      */
-    template <class Evaluation, class FluidState>
+    template <class Evaluation, class FluidState, class ...Args>
     static Evaluation relpermOilInOilGasSystem(const Params& params,
                                                const FluidState& fluidState)
     {
-        OPM_TIMEFUNCTION_LOCAL();
+        OPM_TIMEFUNCTION_LOCAL(Subsystem::SatProps);
         const Evaluation sw =
             max(Evaluation{ params.Swl() },
                      decay<Evaluation>(fluidState.saturation(waterPhaseIdx)));
@@ -429,17 +436,17 @@ public:
         const Evaluation sg = decay<Evaluation>(fluidState.saturation(gasPhaseIdx));
         const Evaluation So_go = 1.0 - (sg + sw);
 
-        return GasOilMaterialLaw::twoPhaseSatKrw(params.gasOilParams(), So_go);
+        return GasOilMaterialLaw::template twoPhaseSatKrw<Evaluation, Args...>(params.gasOilParams(), So_go);
     }
 
     /*!
      * \brief The relative permeability of oil in oil/water system.
      */
-    template <class Evaluation, class FluidState>
+    template <class Evaluation, class FluidState, class ...Args>
     static Evaluation relpermOilInOilWaterSystem(const Params& params,
                                                  const FluidState& fluidState)
     {
-        OPM_TIMEFUNCTION_LOCAL();
+        OPM_TIMEFUNCTION_LOCAL(Subsystem::SatProps);
         const Evaluation sw =
             max(Evaluation{ params.Swl() },
                      decay<Evaluation>(fluidState.saturation(waterPhaseIdx)));
@@ -447,7 +454,7 @@ public:
         const Evaluation sg = decay<Evaluation>(fluidState.saturation(gasPhaseIdx));
         const Evaluation Sw_ow = sg + sw;
 
-        return OilWaterMaterialLaw::twoPhaseSatKrn(params.oilWaterParams(), Sw_ow);
+        return OilWaterMaterialLaw::template twoPhaseSatKrn<Evaluation, Args...>(params.oilWaterParams(), Sw_ow);
     }
 
     /*!
@@ -460,21 +467,25 @@ public:
     template <class FluidState>
     static bool updateHysteresis(Params& params, const FluidState& fluidState)
     {
-        const Scalar Swco = params.Swl();
-        const Scalar sw = clampSaturation(fluidState, waterPhaseIdx);
-        const Scalar So = clampSaturation(fluidState, oilPhaseIdx);
-        const Scalar sg = clampSaturation(fluidState, gasPhaseIdx);
-        bool owChanged = params.oilWaterParams().update(/*pcSw=*/sw, /*krwSw=*/sw, /*krnSw=*/1 - So);
-        bool gochanged = params.gasOilParams().update(/*pcSw=*/  So,
-                                                          /*krwSw=*/ So,
-                                                          /*krnSw=*/ 1.0 - Swco - sg);
-        return owChanged || gochanged;
+        if constexpr (Traits::enableHysteresis) {
+            const Scalar Swco = params.Swl();
+            const Scalar sw = clampSaturation(fluidState, waterPhaseIdx);
+            const Scalar So = clampSaturation(fluidState, oilPhaseIdx);
+            const Scalar sg = clampSaturation(fluidState, gasPhaseIdx);
+            bool owChanged = params.oilWaterParams().update(/*pcSw=*/sw, /*krwSw=*/sw, /*krnSw=*/1 - So);
+            bool gochanged = params.gasOilParams().update(/*pcSw=*/So,
+                                                          /*krwSw=*/So,
+                                                          /*krnSw=*/1.0 - Swco - sg);
+            return owChanged || gochanged;
+        } else {
+            return false;
+        }
     }
 
     template <class FluidState>
     static Scalar clampSaturation(const FluidState& fluidState, const int phaseIndex)
     {
-        OPM_TIMEFUNCTION_LOCAL();
+        OPM_TIMEFUNCTION_LOCAL(Subsystem::SatProps);
         const auto sat = scalarValue(fluidState.saturation(phaseIndex));
         return std::clamp(sat, Scalar{0.0}, Scalar{1.0});
     }
