@@ -31,20 +31,15 @@
 #include <opm/input/eclipse/EclipseState/Grid/SatfuncPropertyInitializers.hpp>
 
 #include <opm/material/fluidmatrixinteractions/EclEpsGridProperties.hpp>
+#include <opm/material/fluidmatrixinteractions/EclMaterialLawInitParams.hpp>
 #include <opm/material/fluidstates/SimpleModularFluidState.hpp>
 
 #include <algorithm>
 
-namespace Opm {
+namespace Opm::EclMaterialLaw {
 
 template<class TraitsT>
-EclMaterialLawManager<TraitsT>::EclMaterialLawManager() = default;
-
-template<class TraitsT>
-EclMaterialLawManager<TraitsT>::~EclMaterialLawManager() = default;
-
-template<class TraitsT>
-void EclMaterialLawManager<TraitsT>::
+void Manager<TraitsT>::
 initFromState(const EclipseState& eclState)
 {
     // get the number of saturation regions and the number of cells in the deck
@@ -52,23 +47,14 @@ initFromState(const EclipseState& eclState)
     const size_t numSatRegions = runspec.tabdims().getNumSatTables();
 
     const auto& ph = runspec.phases();
-    this->hasGas = ph.active(Phase::GAS);
-    this->hasOil = ph.active(Phase::OIL);
-    this->hasWater = ph.active(Phase::WATER);
+    this->hasGas_ = ph.active(Phase::GAS);
+    this->hasOil_ = ph.active(Phase::OIL);
+    this->hasWater_ = ph.active(Phase::WATER);
 
     readGlobalEpsOptions_(eclState);
     readGlobalHysteresisOptions_(eclState);
     readGlobalThreePhaseOptions_(runspec);
 
-    // Read the end point scaling configuration (once per run).
-    gasOilConfig_ = std::make_shared<EclEpsConfig>();
-    oilWaterConfig_ = std::make_shared<EclEpsConfig>();
-    gasWaterConfig_ = std::make_shared<EclEpsConfig>();
-    gasOilConfig_->initFromState(eclState, EclTwoPhaseSystemType::GasOil);
-    oilWaterConfig_->initFromState(eclState, EclTwoPhaseSystemType::OilWater);
-    gasWaterConfig_->initFromState(eclState, EclTwoPhaseSystemType::GasWater);
-
-
     const auto& tables = eclState.getTableManager();
 
     {
@@ -85,7 +71,7 @@ initFromState(const EclipseState& eclState)
                                return table.eta;
                            });
         }
-        
+
         const auto& ppcwmaxTables = tables.getPpcwmax();
         this->enablePpcwmax_ = !ppcwmaxTables.empty();
 
@@ -105,7 +91,7 @@ initFromState(const EclipseState& eclState)
 
     this->unscaledEpsInfo_.resize(numSatRegions);
 
-    if (this->hasGas + this->hasOil + this->hasWater == 1) {
+    if (this->hasGas_ + this->hasOil_ + this->hasWater_ == 1) {
         // Single-phase simulation.  Special case.  Nothing to do here.
         return;
     }
@@ -135,19 +121,21 @@ initFromState(const EclipseState& eclState)
 }
 
 template<class TraitsT>
-void EclMaterialLawManager<TraitsT>::
+void Manager<TraitsT>::
 initParamsForElements(const EclipseState& eclState, size_t numCompressedElems,
                       const std::function<std::vector<int>(const FieldPropsManager&, const std::string&, bool)>& fieldPropIntOnLeafAssigner,
                       const std::function<unsigned(unsigned)>& lookupIdxOnLevelZeroAssigner)
 {
-    InitParams initParams {*this, eclState, numCompressedElems};
+    InitParams<Traits> initParams {*this, eclState, numCompressedElems};
     initParams.run(fieldPropIntOnLeafAssigner, lookupIdxOnLevelZeroAssigner);
+    params_ = std::move(initParams.params_);
 }
 
 // TODO: Better (proper?) handling of mixed wettability systems - see ecl kw OPTIONS switch 74
 // Note: Without OPTIONS[74] the negative part of the Pcow curve is not scaled
 template<class TraitsT>
-std::pair<typename TraitsT::Scalar, bool> EclMaterialLawManager<TraitsT>::
+std::pair<typename TraitsT::Scalar, bool>
+Manager<TraitsT>::
 applySwatinit(unsigned elemIdx,
               Scalar pcow,
               Scalar Sw)
@@ -157,13 +145,13 @@ applySwatinit(unsigned elemIdx,
         return {Sw, /*newSwatInit*/ true};
     }
 
-    auto& elemScaledEpsInfo = oilWaterScaledEpsInfoDrainage_[elemIdx];
+    auto& elemScaledEpsInfo = params_.oilWaterScaledEpsInfoDrainage[elemIdx];
     if (Sw <= elemScaledEpsInfo.Swl)
         Sw = elemScaledEpsInfo.Swl;
 
     // specify a fluid state which only stores the saturations
     using FluidState = SimpleModularFluidState<Scalar,
-                                                numPhases,
+                                                TraitsT::numPhases,
                                                 /*numComponents=*/0,
                                                 /*FluidSystem=*/void, /* -> don't care */
                                                 /*storePressure=*/false,
@@ -175,9 +163,9 @@ applySwatinit(unsigned elemIdx,
                                                 /*storeViscosity=*/false,
                                                 /*storeEnthalpy=*/false>;
     FluidState fs;
-    fs.setSaturation(waterPhaseIdx, Sw);
-    fs.setSaturation(gasPhaseIdx, 0);
-    fs.setSaturation(oilPhaseIdx, 0);
+    fs.setSaturation(TraitsT::wettingPhaseIdx, Sw);
+    fs.setSaturation(TraitsT::gasPhaseIdx, 0);
+    fs.setSaturation(TraitsT::nonWettingPhaseIdx, 0);
     std::array<Scalar, numPhases> pc = { 0 };
     MaterialLaw::capillaryPressures(pc, materialLawParams(elemIdx), fs);
     Scalar pcowAtSw = pc[oilPhaseIdx] - pc[waterPhaseIdx];
@@ -214,36 +202,51 @@ applySwatinit(unsigned elemIdx,
 
     auto& elemEclEpsScalingPoints = oilWaterScaledEpsPointsDrainage(elemIdx);
     elemEclEpsScalingPoints.init(elemScaledEpsInfo,
-                                    *oilWaterEclEpsConfig_,
-                                    EclTwoPhaseSystemType::OilWater);
+                                 oilWaterConfig_,
+                                 EclTwoPhaseSystemType::OilWater);
 
     return {Sw, newSwatInit};
 }
 
 template<class TraitsT>
 void
-EclMaterialLawManager<TraitsT>::applyRestartSwatInit(const unsigned elemIdx,
-                                                     const Scalar   maxPcow)
+Manager<TraitsT>::
+applyRestartSwatInit(const unsigned elemIdx,
+                     const Scalar   maxPcow)
 {
     // Maximum capillary pressure adjusted from SWATINIT data.
 
     auto& elemScaledEpsInfo =
-        this->oilWaterScaledEpsInfoDrainage_[elemIdx];
+        this->params_.oilWaterScaledEpsInfoDrainage[elemIdx];
 
     elemScaledEpsInfo.maxPcow = maxPcow;
 
     this->oilWaterScaledEpsPointsDrainage(elemIdx)
         .init(elemScaledEpsInfo,
-              *this->oilWaterEclEpsConfig_,
+              this->oilWaterConfig_,
               EclTwoPhaseSystemType::OilWater);
 }
 
+namespace
+{
+// Helper to get the eps level parameters.
+template <class TraitsT, class MaybeHystParams>
+auto& getDrainageParams(MaybeHystParams& p)
+{
+    if constexpr(TraitsT::enableHysteresis) {
+        return p.drainageParams();
+    } else {
+        return p;
+    }
+}
+} // anon namespace
+
 template<class TraitsT>
-const typename EclMaterialLawManager<TraitsT>::MaterialLawParams&
-EclMaterialLawManager<TraitsT>::
+const typename Manager<TraitsT>::MaterialLawParams&
+Manager<TraitsT>::
 connectionMaterialLawParams(unsigned satRegionIdx, unsigned elemIdx) const
 {
-    MaterialLawParams& mlp = const_cast<MaterialLawParams&>(materialLawParams_[elemIdx]);
+    MaterialLawParams& mlp = const_cast<MaterialLawParams&>(params_.materialLawParams[elemIdx]);
 
     if (enableHysteresis())
         OpmLog::warning("Warning: Using non-default satnum regions for connection is not tested in combination with hysteresis");
@@ -255,45 +258,45 @@ connectionMaterialLawParams(unsigned satRegionIdx, unsigned elemIdx) const
     case EclMultiplexerApproach::Stone1: {
         auto& realParams = mlp.template getRealParams<EclMultiplexerApproach::Stone1>();
 
-        realParams.oilWaterParams().drainageParams().setUnscaledPoints(oilWaterUnscaledPointsVector_[satRegionIdx]);
-        realParams.oilWaterParams().drainageParams().setEffectiveLawParams(oilWaterEffectiveParamVector_[satRegionIdx]);
-        realParams.gasOilParams().drainageParams().setUnscaledPoints(gasOilUnscaledPointsVector_[satRegionIdx]);
-        realParams.gasOilParams().drainageParams().setEffectiveLawParams(gasOilEffectiveParamVector_[satRegionIdx]);
+        getDrainageParams<Traits>(realParams.oilWaterParams()).setUnscaledPoints(params_.oilWaterUnscaledPointsVector[satRegionIdx]);
+        getDrainageParams<Traits>(realParams.oilWaterParams()).setEffectiveLawParams(params_.oilWaterEffectiveParamVector[satRegionIdx]);
+        getDrainageParams<Traits>(realParams.gasOilParams()).setUnscaledPoints(params_.gasOilUnscaledPointsVector[satRegionIdx]);
+        getDrainageParams<Traits>(realParams.gasOilParams()).setEffectiveLawParams(params_.gasOilEffectiveParamVector[satRegionIdx]);
 //            if (enableHysteresis()) {
-//                realParams.oilWaterParams().imbibitionParams().setUnscaledPoints(oilWaterUnscaledPointsVector_[impRegionIdx]);
-//                realParams.oilWaterParams().imbibitionParams().setEffectiveLawParams(oilWaterEffectiveParamVector_[impRegionIdx]);
-//                realParams.gasOilParams().imbibitionParams().setUnscaledPoints(gasOilUnscaledPointsVector_[impRegionIdx]);
-//                realParams.gasOilParams().imbibitionParams().setEffectiveLawParams(gasOilEffectiveParamVector_[impRegionIdx]);
+//                getImbParams(realParams.oilWaterParams()).setUnscaledPoints(oilWaterUnscaledPointsVector_[impRegionIdx]);
+//                getImbParams(realParams.oilWaterParams()).setEffectiveLawParams(oilWaterEffectiveParamVector_[impRegionIdx]);
+//                getImbParams(realParams.gasOilParams()).setUnscaledPoints(gasOilUnscaledPointsVector_[impRegionIdx]);
+//                getImbParams(realParams.gasOilParams()).setEffectiveLawParams(gasOilEffectiveParamVector_[impRegionIdx]);
 //            }
     }
         break;
 
     case EclMultiplexerApproach::Stone2: {
         auto& realParams = mlp.template getRealParams<EclMultiplexerApproach::Stone2>();
-        realParams.oilWaterParams().drainageParams().setUnscaledPoints(oilWaterUnscaledPointsVector_[satRegionIdx]);
-        realParams.oilWaterParams().drainageParams().setEffectiveLawParams(oilWaterEffectiveParamVector_[satRegionIdx]);
-        realParams.gasOilParams().drainageParams().setUnscaledPoints(gasOilUnscaledPointsVector_[satRegionIdx]);
-        realParams.gasOilParams().drainageParams().setEffectiveLawParams(gasOilEffectiveParamVector_[satRegionIdx]);
+        getDrainageParams<Traits>(realParams.oilWaterParams()).setUnscaledPoints(params_.oilWaterUnscaledPointsVector[satRegionIdx]);
+        getDrainageParams<Traits>(realParams.oilWaterParams()).setEffectiveLawParams(params_.oilWaterEffectiveParamVector[satRegionIdx]);
+        getDrainageParams<Traits>(realParams.gasOilParams()).setUnscaledPoints(params_.gasOilUnscaledPointsVector[satRegionIdx]);
+        getDrainageParams<Traits>(realParams.gasOilParams()).setEffectiveLawParams(params_.gasOilEffectiveParamVector[satRegionIdx]);
 //            if (enableHysteresis()) {
-//                realParams.oilWaterParams().imbibitionParams().setUnscaledPoints(oilWaterUnscaledPointsVector_[impRegionIdx]);
-//                realParams.oilWaterParams().imbibitionParams().setEffectiveLawParams(oilWaterEffectiveParamVector_[impRegionIdx]);
-//                realParams.gasOilParams().imbibitionParams().setUnscaledPoints(gasOilUnscaledPointsVector_[impRegionIdx]);
-//                realParams.gasOilParams().imbibitionParams().setEffectiveLawParams(gasOilEffectiveParamVector_[impRegionIdx]);
+//                getImbParams(realParams.oilWaterParams()).setUnscaledPoints(oilWaterUnscaledPointsVector_[impRegionIdx]);
+//                getImbParams(realParams.oilWaterParams()).setEffectiveLawParams(oilWaterEffectiveParamVector_[impRegionIdx]);
+//                getImbParams(realParams.gasOilParams()).setUnscaledPoints(gasOilUnscaledPointsVector_[impRegionIdx]);
+//                getImbParams(realParams.gasOilParams()).setEffectiveLawParams(gasOilEffectiveParamVector_[impRegionIdx]);
 //            }
     }
         break;
 
     case EclMultiplexerApproach::Default: {
         auto& realParams = mlp.template getRealParams<EclMultiplexerApproach::Default>();
-        realParams.oilWaterParams().drainageParams().setUnscaledPoints(oilWaterUnscaledPointsVector_[satRegionIdx]);
-        realParams.oilWaterParams().drainageParams().setEffectiveLawParams(oilWaterEffectiveParamVector_[satRegionIdx]);
-        realParams.gasOilParams().drainageParams().setUnscaledPoints(gasOilUnscaledPointsVector_[satRegionIdx]);
-        realParams.gasOilParams().drainageParams().setEffectiveLawParams(gasOilEffectiveParamVector_[satRegionIdx]);
+        getDrainageParams<Traits>(realParams.oilWaterParams()).setUnscaledPoints(params_.oilWaterUnscaledPointsVector[satRegionIdx]);
+        getDrainageParams<Traits>(realParams.oilWaterParams()).setEffectiveLawParams(params_.oilWaterEffectiveParamVector[satRegionIdx]);
+        getDrainageParams<Traits>(realParams.gasOilParams()).setUnscaledPoints(params_.gasOilUnscaledPointsVector[satRegionIdx]);
+        getDrainageParams<Traits>(realParams.gasOilParams()).setEffectiveLawParams(params_.gasOilEffectiveParamVector[satRegionIdx]);
 //            if (enableHysteresis()) {
-//                realParams.oilWaterParams().imbibitionParams().setUnscaledPoints(oilWaterUnscaledPointsVector_[impRegionIdx]);
-//                realParams.oilWaterParams().imbibitionParams().setEffectiveLawParams(oilWaterEffectiveParamVector_[impRegionIdx]);
-//                realParams.gasOilParams().imbibitionParams().setUnscaledPoints(gasOilUnscaledPointsVector_[impRegionIdx]);
-//                realParams.gasOilParams().imbibitionParams().setEffectiveLawParams(gasOilEffectiveParamVector_[impRegionIdx]);
+//                getImbParams(realParams.oilWaterParams()).setUnscaledPoints(oilWaterUnscaledPointsVector_[impRegionIdx]);
+//                getImbParams(realParams.oilWaterParams()).setEffectiveLawParams(oilWaterEffectiveParamVector_[impRegionIdx]);
+//                getImbParams(realParams.gasOilParams()).setUnscaledPoints(gasOilUnscaledPointsVector_[impRegionIdx]);
+//                getImbParams(realParams.gasOilParams()).setEffectiveLawParams(gasOilEffectiveParamVector_[impRegionIdx]);
 //            }
     }
         break;
@@ -301,22 +304,22 @@ connectionMaterialLawParams(unsigned satRegionIdx, unsigned elemIdx) const
     case EclMultiplexerApproach::TwoPhase: {
         auto& realParams = mlp.template getRealParams<EclMultiplexerApproach::TwoPhase>();
         if (realParams.approach() == EclTwoPhaseApproach::GasOil) {
-            realParams.gasOilParams().drainageParams().setUnscaledPoints(gasOilUnscaledPointsVector_[satRegionIdx]);
-            realParams.gasOilParams().drainageParams().setEffectiveLawParams(gasOilEffectiveParamVector_[satRegionIdx]);
+            getDrainageParams<Traits>(realParams.gasOilParams()).setUnscaledPoints(params_.gasOilUnscaledPointsVector[satRegionIdx]);
+            getDrainageParams<Traits>(realParams.gasOilParams()).setEffectiveLawParams(params_.gasOilEffectiveParamVector[satRegionIdx]);
         }
         else if (realParams.approach() == EclTwoPhaseApproach::GasWater) {
-            realParams.gasWaterParams().drainageParams().setUnscaledPoints(gasWaterUnscaledPointsVector_[satRegionIdx]);
-            realParams.gasWaterParams().drainageParams().setEffectiveLawParams(gasWaterEffectiveParamVector_[satRegionIdx]);
+            getDrainageParams<Traits>(realParams.gasWaterParams()).setUnscaledPoints(params_.gasWaterUnscaledPointsVector[satRegionIdx]);
+            getDrainageParams<Traits>(realParams.gasWaterParams()).setEffectiveLawParams(params_.gasWaterEffectiveParamVector[satRegionIdx]);
         }
         else if (realParams.approach() == EclTwoPhaseApproach::OilWater) {
-            realParams.oilWaterParams().drainageParams().setUnscaledPoints(oilWaterUnscaledPointsVector_[satRegionIdx]);
-            realParams.oilWaterParams().drainageParams().setEffectiveLawParams(oilWaterEffectiveParamVector_[satRegionIdx]);
+            getDrainageParams<Traits>(realParams.oilWaterParams()).setUnscaledPoints(params_.oilWaterUnscaledPointsVector[satRegionIdx]);
+            getDrainageParams<Traits>(realParams.oilWaterParams()).setEffectiveLawParams(params_.oilWaterEffectiveParamVector[satRegionIdx]);
         }
 //            if (enableHysteresis()) {
-//                realParams.oilWaterParams().imbibitionParams().setUnscaledPoints(oilWaterUnscaledPointsVector_[impRegionIdx]);
-//                realParams.oilWaterParams().imbibitionParams().setEffectiveLawParams(oilWaterEffectiveParamVector_[impRegionIdx]);
-//                realParams.gasOilParams().imbibitionParams().setUnscaledPoints(gasOilUnscaledPointsVector_[impRegionIdx]);
-//                realParams.gasOilParams().imbibitionParams().setEffectiveLawParams(gasOilEffectiveParamVector_[impRegionIdx]);
+//                getImbParams(realParams.oilWaterParams()).setUnscaledPoints(oilWaterUnscaledPointsVector_[impRegionIdx]);
+//                getImbParams(realParams.oilWaterParams()).setEffectiveLawParams(oilWaterEffectiveParamVector_[impRegionIdx]);
+//                getImbParams(realParams.gasOilParams()).setUnscaledPoints(gasOilUnscaledPointsVector_[impRegionIdx]);
+//                getImbParams(realParams.gasOilParams()).setEffectiveLawParams(gasOilEffectiveParamVector_[impRegionIdx]);
 //            }
     }
         break;
@@ -324,25 +327,25 @@ connectionMaterialLawParams(unsigned satRegionIdx, unsigned elemIdx) const
     default:
         throw std::logic_error("Enum value for material approach unknown!");
     }
-
     return mlp;
 }
 
 template<class TraitsT>
-int EclMaterialLawManager<TraitsT>::
+int
+Manager<TraitsT>::
 getKrnumSatIdx(unsigned elemIdx, FaceDir::DirEnum facedir) const
 {
     using Dir = FaceDir::DirEnum;
     const std::vector<int>* array = nullptr;
     switch(facedir) {
     case Dir::XPlus:
-      array = &krnumXArray_;
+      array = &params_.krnumXArray;
       break;
     case Dir::YPlus:
-      array = &krnumYArray_;
+      array = &params_.krnumYArray;
       break;
     case Dir::ZPlus:
-      array = &krnumZArray_;
+      array = &params_.krnumZArray;
       break;
     default:
       throw std::runtime_error("Unknown face direction");
@@ -351,26 +354,28 @@ getKrnumSatIdx(unsigned elemIdx, FaceDir::DirEnum facedir) const
       return (*array)[elemIdx];
     }
     else {
-      return satnumRegionArray_[elemIdx];
+      return params_.satnumRegionArray[elemIdx];
     }
 }
 
 template<class TraitsT>
-void EclMaterialLawManager<TraitsT>::
+void
+Manager<TraitsT>::
 oilWaterHysteresisParams(Scalar& soMax,
                          Scalar& swMax,
                          Scalar& swMin,
                          unsigned elemIdx) const
 {
-    if (!enableHysteresis())
+    if (!enableHysteresis()) {
         throw std::runtime_error("Cannot get hysteresis parameters if hysteresis not enabled.");
+    }
 
-    const auto& params = materialLawParams(elemIdx);
-    MaterialLaw::oilWaterHysteresisParams(soMax, swMax, swMin, params);
+    MaterialLaw::oilWaterHysteresisParams(soMax, swMax, swMin, materialLawParams(elemIdx));
 }
 
 template<class TraitsT>
-void EclMaterialLawManager<TraitsT>::
+void
+Manager<TraitsT>::
 setOilWaterHysteresisParams(const Scalar& soMax,
                             const Scalar& swMax,
                             const Scalar& swMin,
@@ -379,26 +384,27 @@ setOilWaterHysteresisParams(const Scalar& soMax,
     if (!enableHysteresis())
         throw std::runtime_error("Cannot set hysteresis parameters if hysteresis not enabled.");
 
-    auto& params = materialLawParams(elemIdx);
-    MaterialLaw::setOilWaterHysteresisParams(soMax, swMax, swMin, params);
+    MaterialLaw::setOilWaterHysteresisParams(soMax, swMax, swMin, materialLawParams(elemIdx));
 }
 
 template<class TraitsT>
-void EclMaterialLawManager<TraitsT>::
+void
+Manager<TraitsT>::
 gasOilHysteresisParams(Scalar& sgmax,
                        Scalar& shmax,
                        Scalar& somin,
                        unsigned elemIdx) const
 {
-    if (!enableHysteresis())
+    if (!enableHysteresis()) {
         throw std::runtime_error("Cannot get hysteresis parameters if hysteresis not enabled.");
+    }
 
-    const auto& params = materialLawParams(elemIdx);
-    MaterialLaw::gasOilHysteresisParams(sgmax, shmax, somin, params);
+    MaterialLaw::gasOilHysteresisParams(sgmax, shmax, somin, materialLawParams(elemIdx));
 }
 
 template<class TraitsT>
-void EclMaterialLawManager<TraitsT>::
+void
+Manager<TraitsT>::
 setGasOilHysteresisParams(const Scalar& sgmax,
                           const Scalar& shmax,
                           const Scalar& somin,
@@ -407,35 +413,48 @@ setGasOilHysteresisParams(const Scalar& sgmax,
     if (!enableHysteresis())
         throw std::runtime_error("Cannot set hysteresis parameters if hysteresis not enabled.");
 
-    auto& params = materialLawParams(elemIdx);
-    MaterialLaw::setGasOilHysteresisParams(sgmax, shmax, somin, params);
+    MaterialLaw::setGasOilHysteresisParams(sgmax, shmax, somin, materialLawParams(elemIdx));
 }
 
+namespace
+{
+template<class TraitsT, class MaterialLawParamsT>
+EclEpsScalingPoints<typename TraitsT::Scalar>&
+owsepdHelper(MaterialLawParamsT& mlp)
+{
+    if constexpr(TraitsT::enableHysteresis) {
+        return mlp.oilWaterParams().drainageParams().scaledPoints();
+    } else {
+        return mlp.oilWaterParams().scaledPoints();
+    }
+}
+} // anon namespace
+
 template<class TraitsT>
 EclEpsScalingPoints<typename TraitsT::Scalar>&
-EclMaterialLawManager<TraitsT>::
+Manager<TraitsT>::
 oilWaterScaledEpsPointsDrainage(unsigned elemIdx)
 {
-    auto& materialParams = materialLawParams_[elemIdx];
+    auto& materialParams = params_.materialLawParams[elemIdx];
     switch (materialParams.approach()) {
     case EclMultiplexerApproach::Stone1: {
         auto& realParams = materialParams.template getRealParams<EclMultiplexerApproach::Stone1>();
-        return realParams.oilWaterParams().drainageParams().scaledPoints();
+        return owsepdHelper<Traits>(realParams);
     }
 
     case EclMultiplexerApproach::Stone2: {
         auto& realParams = materialParams.template getRealParams<EclMultiplexerApproach::Stone2>();
-        return realParams.oilWaterParams().drainageParams().scaledPoints();
+        return owsepdHelper<Traits>(realParams);
     }
 
     case EclMultiplexerApproach::Default: {
         auto& realParams = materialParams.template getRealParams<EclMultiplexerApproach::Default>();
-        return realParams.oilWaterParams().drainageParams().scaledPoints();
+        return owsepdHelper<Traits>(realParams);
     }
 
     case EclMultiplexerApproach::TwoPhase: {
         auto& realParams = materialParams.template getRealParams<EclMultiplexerApproach::TwoPhase>();
-        return realParams.oilWaterParams().drainageParams().scaledPoints();
+        return owsepdHelper<Traits>(realParams);
     }
     default:
         throw std::logic_error("Enum value for material approach unknown!");
@@ -443,50 +462,55 @@ oilWaterScaledEpsPointsDrainage(unsigned elemIdx)
 }
 
 template<class TraitsT>
-const typename EclMaterialLawManager<TraitsT>::MaterialLawParams& EclMaterialLawManager<TraitsT>::
+const typename Manager<TraitsT>::MaterialLawParams&
+Manager<TraitsT>::
 materialLawParamsFunc_(unsigned elemIdx, FaceDir::DirEnum facedir) const
 {
     using Dir = FaceDir::DirEnum;
-    if (dirMaterialLawParams_) {
+    if (params_.dirMaterialLawParams) {
         switch(facedir) {
             case Dir::XMinus:
             case Dir::XPlus:
-                return dirMaterialLawParams_->materialLawParamsX_[elemIdx];
+                return params_.dirMaterialLawParams->materialLawParamsX_[elemIdx];
             case Dir::YMinus:
             case Dir::YPlus:
-                return dirMaterialLawParams_->materialLawParamsY_[elemIdx];
+                return params_.dirMaterialLawParams->materialLawParamsY_[elemIdx];
             case Dir::ZMinus:
             case Dir::ZPlus:
-                return dirMaterialLawParams_->materialLawParamsZ_[elemIdx];
+                return params_.dirMaterialLawParams->materialLawParamsZ_[elemIdx];
             default:
                 throw std::runtime_error("Unexpected face direction");
         }
     }
     else {
-        return materialLawParams_[elemIdx];
+        return params_.materialLawParams[elemIdx];
     }
 }
 
 template<class TraitsT>
-void EclMaterialLawManager<TraitsT>::
+void
+Manager<TraitsT>::
 readGlobalEpsOptions_(const EclipseState& eclState)
 {
-    oilWaterEclEpsConfig_ = std::make_shared<EclEpsConfig>();
-    oilWaterEclEpsConfig_->initFromState(eclState, EclTwoPhaseSystemType::OilWater);
-
     enableEndPointScaling_ = eclState.getTableManager().hasTables("ENKRVD");
+
+    // Read the end point scaling configuration (once per run).
+    gasOilConfig_.initFromState(eclState, EclTwoPhaseSystemType::GasOil);
+    oilWaterConfig_.initFromState(eclState, EclTwoPhaseSystemType::OilWater);
+    gasWaterConfig_.initFromState(eclState, EclTwoPhaseSystemType::GasWater);
 }
 
 template<class TraitsT>
-void EclMaterialLawManager<TraitsT>::
+void
+Manager<TraitsT>::
 readGlobalHysteresisOptions_(const EclipseState& state)
 {
-    hysteresisConfig_ = std::make_shared<EclHysteresisConfig>();
-    hysteresisConfig_->initFromState(state.runspec());
+    hysteresisConfig_.initFromState(state.runspec());
 }
 
 template<class TraitsT>
-void EclMaterialLawManager<TraitsT>::
+void
+Manager<TraitsT>::
 readGlobalThreePhaseOptions_(const Runspec& runspec)
 {
     bool gasEnabled = runspec.phases().active(Phase::GAS);
@@ -523,10 +547,11 @@ readGlobalThreePhaseOptions_(const Runspec& runspec)
     }
 }
 
+template class Manager<ThreePhaseMaterialTraits<double,0,1,2,true,true>>;
+template class Manager<ThreePhaseMaterialTraits<float,0,1,2,true,true>>;
+template class Manager<ThreePhaseMaterialTraits<double,2,0,1,true,true>>;
+template class Manager<ThreePhaseMaterialTraits<float,2,0,1,true,true>>;
+template class Manager<ThreePhaseMaterialTraits<double,0,1,2,false,true>>;
+template class Manager<ThreePhaseMaterialTraits<float,0,1,2,false,true>>;
 
-template class EclMaterialLawManager<ThreePhaseMaterialTraits<double,0,1,2>>;
-template class EclMaterialLawManager<ThreePhaseMaterialTraits<float,0,1,2>>;
-template class EclMaterialLawManager<ThreePhaseMaterialTraits<double,2,0,1>>;
-template class EclMaterialLawManager<ThreePhaseMaterialTraits<float,2,0,1>>;
-
-} // namespace Opm
+} // namespace Opm::EclMaterialLaw
