@@ -27,15 +27,11 @@
 #ifndef OPM_BASE_FLUID_SYSTEM_HPP
 #define OPM_BASE_FLUID_SYSTEM_HPP
 
-#include <opm/common/utility/Demangle.hpp>
+#include <opm/common/utility/DemangledType.hpp>
 
 #include <stdexcept>
 #include <string_view>
 
-#include "NullParameterCache.hpp"
-
-#include <typeinfo>
-
 namespace Opm {
 
 /*!
@@ -300,7 +296,7 @@ private:
     static std::string not_implemented(const std::string_view method)
     {
         return "Not implemented: The fluid system '" +
-               demangle(typeid(Implementation).name()) +
+               getDemangledType<Implementation>() +
                "'  does not provide a " + method.data() + "() method!";
     }
 };
