@@ -35,6 +35,7 @@ copyright holders.
 #include <opm/material/components/H2.hpp>
 #include <opm/material/common/UniformTabulated2DFunction.hpp>
 #include <opm/material/common/Valgrind.hpp>
+#include <opm/material/fluidsystems/BlackOilFunctions.hpp>
 
 #include <cstddef>
 #include <vector>
@@ -261,6 +262,27 @@ public:
              / brineReferenceDensity_[regionIdx];
     }
 
+    /*!
+     * \brief Returns the formation volume factor [-] and viscosity [Pa s] of the fluid phase.
+     */
+    template <class FluidState, class LhsEval = typename FluidState::Scalar>
+    std::pair<LhsEval, LhsEval>
+    inverseFormationVolumeFactorAndViscosity(const FluidState& fluidState, unsigned regionIdx)
+    {
+        // Deal with the possibility that we are in a two-phase H2STORE with OIL and GAS as phases.
+        const bool waterIsActive = fluidState.phaseIsActive(FluidState::waterPhaseIdx);
+        const int myPhaseIdx = waterIsActive ? FluidState::waterPhaseIdx : FluidState::oilPhaseIdx;
+        const LhsEval& Rsw = waterIsActive ? decay<LhsEval>(fluidState.Rsw()) : decay<LhsEval>(fluidState.Rs());
+
+        const LhsEval& T = decay<LhsEval>(fluidState.temperature(myPhaseIdx));
+        const LhsEval& p = decay<LhsEval>(fluidState.pressure(myPhaseIdx));
+        const LhsEval& saltConcentration
+            = BlackOil::template getSaltConcentration_<FluidState, LhsEval>(fluidState, regionIdx);
+        // TODO: The viscosity does not yet depend on the composition
+        return { this->inverseFormationVolumeFactor(regionIdx, T, p, Rsw, saltConcentration) ,
+                this->saturatedViscosity(regionIdx, T, p, saltConcentration) };
+    }
+
     /*!
     * \brief Returns the formation volume factor [-] of brine saturated
     *        with H2 at a given pressure.
@@ -459,8 +481,8 @@ private:
         }
 
         // calculate individual contribution to density
-        const LhsEval& rho_brine = Brine::liquidDensity(T, pl, salinity, extrapolate);
         const LhsEval& rho_pure = H2O::liquidDensity(T, pl, extrapolate);
+        const LhsEval& rho_brine = Brine::liquidDensity(T, pl, salinity, rho_pure);
         const LhsEval& rho_lH2 = liquidDensityWaterH2_(T, pl, xlH2);
         const LhsEval& contribH2 = rho_lH2 - rho_pure;
 
