@@ -20,6 +20,8 @@
 #ifndef OPM_OUTPUT_WELLS_HPP
 #define OPM_OUTPUT_WELLS_HPP
 
+#include <opm/common/ErrorMacros.hpp>
+#include <opm/common/OpmLog/OpmLog.hpp>
 #include <opm/output/data/GuideRateValue.hpp>
 #include <opm/input/eclipse/Schedule/Well/WellEnums.hpp>
 
@@ -75,7 +77,8 @@ namespace Opm { namespace data {
                 oxygen           = (1 << 21),
                 urea             = (1 << 22),
                 vaporized_water  = (1 << 23),
-                mass_gas         = (1 << 24)
+                mass_gas         = (1 << 24),
+                mass_wat         = (1 << 25)
             };
 
             using enum_size = std::underlying_type< opt >::type;
@@ -137,6 +140,7 @@ namespace Opm { namespace data {
                 serializer(urea);
                 serializer(vaporized_water);
                 serializer(mass_gas);
+                serializer(mass_wat);
             }
 
             static Rates serializationTestObject()
@@ -166,6 +170,7 @@ namespace Opm { namespace data {
                 rat1.set(opt::urea, 22.0);
                 rat1.set(opt::vaporized_water, 23.0);
                 rat1.set(opt::mass_gas, 24.0);
+                rat1.set(opt::mass_wat, 25.0);
                 rat1.tracer.insert({"test_tracer", 1.0});
 
                 return rat1;
@@ -204,21 +209,25 @@ namespace Opm { namespace data {
             double urea = 0.0;
             double vaporized_water = 0.0;
             double mass_gas = 0.0;
+            double mass_wat = 0.0;
     };
 
     struct ConnectionFiltrate
     {
-        double rate;
-        double total;
-        double skin_factor;
-        double thickness;
-        double perm;
-        double poro;
-        double radius;
-        double area_of_flow;
+        double rate{};
+        double total{};
+        double skin_factor{};
+        double thickness{};
+        double perm{};
+        double poro{};
+        double radius{};
+        double area_of_flow{};
+        double flow_factor{};
+        double fracture_rate{};
 
-        template<class Serializer>
-        void serializeOp(Serializer& serializer) {
+        template <class Serializer>
+        void serializeOp(Serializer& serializer)
+        {
             serializer(rate);
             serializer(total);
             serializer(skin_factor);
@@ -227,24 +236,29 @@ namespace Opm { namespace data {
             serializer(poro);
             serializer(radius);
             serializer(area_of_flow);
+            serializer(flow_factor);
+            serializer(fracture_rate);
         }
 
         bool operator==(const ConnectionFiltrate& filtrate) const
         {
-            return this->rate == filtrate.rate &&
-                   this->total == filtrate.total &&
-                   this->skin_factor == filtrate.skin_factor &&
-                   this->thickness == filtrate.thickness &&
-                   this->perm == filtrate.perm &&
-                   this->poro == filtrate.poro &&
-                   this->radius == filtrate.radius &&
-                   this->area_of_flow == filtrate.area_of_flow;
+            return (this->rate == filtrate.rate)
+                && (this->total == filtrate.total)
+                && (this->skin_factor == filtrate.skin_factor)
+                && (this->thickness == filtrate.thickness)
+                && (this->perm == filtrate.perm)
+                && (this->poro == filtrate.poro)
+                && (this->radius == filtrate.radius)
+                && (this->area_of_flow == filtrate.area_of_flow)
+                && (this->flow_factor == filtrate.flow_factor)
+                && (this->fracture_rate == filtrate.fracture_rate)
+                ;
         }
 
         static ConnectionFiltrate serializationTestObject()
         {
-            return {0.8, 100., -1., 2., 1.e-9,
-                    0.3, 0.05, 0.8};
+            return {0.8, 100.0, -1.0, 2.0, 1.0e-9,
+                    0.3, 0.05, 0.8, 0.1, 0.7};
         }
 
         template <class MessageBufferType>
@@ -254,6 +268,80 @@ namespace Opm { namespace data {
         void read(MessageBufferType& buffer);
     };
 
+    struct ConnectionFracture
+    {
+        double area{};
+        double flux{};
+        double height{};
+        double length{};
+        double WI{};
+        double volume{};
+        double filter_volume{};
+        double avg_width{};
+        double avg_filter_width{};
+
+        template <class Serializer>
+        void serializeOp(Serializer& serializer)
+        {
+            serializer(area);
+            serializer(flux);
+            serializer(height);
+            serializer(length);
+            serializer(WI);
+            serializer(volume);
+            serializer(filter_volume);
+            serializer(avg_width);
+            serializer(avg_filter_width);
+        }
+
+        bool operator==(const ConnectionFracture& fraccon) const
+        {
+            return (this->area == fraccon.area)
+                && (this->flux == fraccon.flux)
+                && (this->height == fraccon.height)
+                && (this->length == fraccon.length)
+                && (this->WI == fraccon.WI)
+                && (this->volume == fraccon.volume)
+                && (this->filter_volume == fraccon.filter_volume)
+                && (this->avg_width == fraccon.avg_width)
+                && (this->avg_filter_width == fraccon.avg_filter_width)
+                ;
+        }
+
+        static ConnectionFracture serializationTestObject()
+        {
+            return {0.8, 100.0, 1.3, 1.4, 10.0, 4.0, 0.4, 0.5, 0.05};
+        }
+
+        template <class MessageBufferType>
+        void write(MessageBufferType& buffer) const
+        {
+            buffer.write(this->area);
+            buffer.write(this->flux);
+            buffer.write(this->height);
+            buffer.write(this->length);
+            buffer.write(this->WI);
+            buffer.write(this->volume);
+            buffer.write(this->filter_volume);
+            buffer.write(this->avg_width);
+            buffer.write(this->avg_filter_width);
+        }
+
+        template <class MessageBufferType>
+        void read(MessageBufferType& buffer)
+        {
+            buffer.read(this->area);
+            buffer.read(this->flux);
+            buffer.read(this->height);
+            buffer.read(this->length);
+            buffer.read(this->WI);
+            buffer.read(this->volume);
+            buffer.read(this->filter_volume);
+            buffer.read(this->avg_width);
+            buffer.read(this->avg_filter_width);
+        }
+    };
+
     /// Connection Level Fracturing Statistics
     struct ConnectionFracturing
     {
@@ -428,8 +516,12 @@ namespace Opm { namespace data {
         double d_factor{};
         double compact_mult{1.0}; // Rock compaction transmissibility multiplier (ROCKTAB)
 
+        int lgr_grid{0}; // LGR grid index, 0 if not in LGR
+
         ConnectionFiltrate filtrate{};
 
+        ConnectionFracture fracture{};
+
         /// Connection level fracturing statistics.
         ConnectionFracturing fract{};
 
@@ -446,7 +538,9 @@ namespace Opm { namespace data {
                 && (trans_factor == conn2.trans_factor)
                 && (d_factor == conn2.d_factor)
                 && (compact_mult == conn2.compact_mult)
+                && (lgr_grid == conn2.lgr_grid)
                 && (filtrate == conn2.filtrate)
+                && (fracture == conn2.fracture)
                 && (this->fract == conn2.fract)
                 ;
         }
@@ -472,7 +566,9 @@ namespace Opm { namespace data {
             serializer(trans_factor);
             serializer(d_factor);
             serializer(compact_mult);
+            serializer(lgr_grid);
             serializer(filtrate);
+            serializer(fracture);
             serializer(this->fract);
         }
 
@@ -482,7 +578,9 @@ namespace Opm { namespace data {
                 1, Rates::serializationTestObject(),
                 2.0, 3.0, 4.0, 5.0,
                 6.0, 7.0, 8.0, 9.0, 0.987,
+                3, // lgr_grid
                 ConnectionFiltrate::serializationTestObject(),
+                ConnectionFracture::serializationTestObject(),
                 ConnectionFracturing::serializationTestObject()
             };
         }
@@ -1055,6 +1153,11 @@ namespace Opm { namespace data {
                 ;
         }
 
+        bool operator!=(const Well& well2) const
+        {
+            return !(*this == well2);
+        }
+
         template<class Serializer>
         void serializeOp(Serializer& serializer)
         {
@@ -1147,7 +1250,14 @@ namespace Opm { namespace data {
                 buffer.read(name);
                 Well well;
                 well.read(buffer);
-                this->emplace(name, well);
+                auto result = this->emplace(name, well);
+                // In case there was already an entry for the well we want to insert, then result.second == false.
+                // Then we check if this entry is the same as the one we want to insert.
+                if (!result.second && result.first->second != well) {
+                    OPM_THROW(std::runtime_error, "Received different output data for well " + name + " from more than one process, the output of this simulation will be wrong!");
+                } else if (!result.second) {
+                    OpmLog::warning("Received consistently duplicated output data for well " + name + " from more than one process - this might be problematic!");
+                }
             }
         }
 
@@ -1290,7 +1400,8 @@ namespace Opm { namespace data {
              oxygen == rate.oxygen &&
              urea == rate.urea &&
              vaporized_water == rate.vaporized_water &&
-             mass_gas == rate.mass_gas;
+             mass_gas == rate.mass_gas &&
+             mass_wat == rate.mass_wat;
     }
 
 
@@ -1331,6 +1442,7 @@ namespace Opm { namespace data {
             case opt::urea: return this->urea;
             case opt::vaporized_water: return this->vaporized_water;
             case opt::mass_gas: return this->mass_gas;
+            case opt::mass_wat: return this->mass_wat;
         }
 
         throw std::invalid_argument(
@@ -1417,6 +1529,7 @@ namespace Opm { namespace data {
             buffer.write(this->urea);
             buffer.write(this->vaporized_water);
             buffer.write(this->mass_gas);
+            buffer.write(this->mass_wat);
     }
 
     template <class MessageBufferType>
@@ -1429,6 +1542,8 @@ namespace Opm { namespace data {
         buffer.write(this->poro);
         buffer.write(this->radius);
         buffer.write(this->area_of_flow);
+        buffer.write(this->flow_factor);
+        buffer.write(this->fracture_rate);
     }
 
     template <class MessageBufferType>
@@ -1444,7 +1559,9 @@ namespace Opm { namespace data {
             buffer.write(this->trans_factor);
             buffer.write(this->d_factor);
             buffer.write(this->compact_mult);
+            buffer.write(this->lgr_grid);
             this->filtrate.write(buffer);
+            this->fracture.write(buffer);
             this->fract.write(buffer);
     }
 
@@ -1596,6 +1713,7 @@ namespace Opm { namespace data {
             buffer.read(this->urea);
             buffer.read(this->vaporized_water);
             buffer.read(this->mass_gas);
+            buffer.read(this->mass_wat);
     }
 
     template <class MessageBufferType>
@@ -1608,6 +1726,8 @@ namespace Opm { namespace data {
         buffer.read(this->poro);
         buffer.read(this->radius);
         buffer.read(this->area_of_flow);
+        buffer.read(this->flow_factor);
+        buffer.read(this->fracture_rate);
     }
 
    template <class MessageBufferType>
@@ -1623,7 +1743,9 @@ namespace Opm { namespace data {
             buffer.read(this->trans_factor);
             buffer.read(this->d_factor);
             buffer.read(this->compact_mult);
+            buffer.read(this->lgr_grid);
             this->filtrate.read(buffer);
+            this->fracture.read(buffer);
             this->fract.read(buffer);
    }
 
