@@ -91,6 +91,21 @@ namespace Opm::RestartIO {
               std::optional<Helpers::AggregateAquiferData>& aquiferData,
               bool                                          write_double = false);
 
+    // Overloaded function to handle grid containing LGR
+    void save(EclIO::OutputStream::Restart&                 rstFile,
+              int                                           report_step,
+              double                                        seconds_elapsed,
+              std::vector<RestartValue>                     value,
+              const EclipseState&                           es,
+              const EclipseGrid&                            grid,
+              const Schedule&                               schedule,
+              const Action::State&                          action_state,
+              const WellTestState&                          wtest_state,
+              const SummaryState&                           sumState,
+              const UDQState&                               udqState,
+              std::optional<Helpers::AggregateAquiferData>& aquiferData,
+              bool                                          write_double = false);
+
 
     RestartValue load(const std::string&             filename,
                       int                            report_step,
@@ -102,6 +117,7 @@ namespace Opm::RestartIO {
                       const Schedule&                schedule,
                       const std::vector<RestartKey>& extra_keys = {});
 
+
     data::Solution load_solution_only(const std::string&             filename,
                                       int                            report_step,
                                       const std::vector<RestartKey>& solution_keys,
