@@ -28,9 +28,14 @@ namespace Opm { namespace RestartIO { namespace Helpers { namespace VectorItems
         enum index : std::vector<int>::size_type {
             IHead    =  0, // I-location (one-based) of well head
             JHead    =  1, // J-location (one-based) of well head
+                           // IHead and JHead < 0 for completed LGR with
+                           // heads on other LGRs.
             FirstK   =  2, // Layer ID (one-based) of top/first connection
             LastK    =  3, // Layer ID (one-based) of bottom/last connection
             NConn    =  4, // Number of active cells connected to well
+                           // If Well is in amalgamated LGR cells, this is the
+                           // number in all LGR cells.
+
             Group    =  5, // Index (one-based) of well's current group
             WType    =  6, // Well type (producer vs. injector)
             ActWCtrl =  7, // Well's active target control mode (constraint).
@@ -93,6 +98,11 @@ namespace Opm { namespace RestartIO { namespace Helpers { namespace VectorItems
                                        // Example: PEG   = 2 * 3 * 5          =   30
                                        //          PEGDC = 2 * 3 * 5 * 7 * 11 = 2310
 
+
+            LGRIndex = 42,            // Index of LGR cell where well is completed
+                                      // for Globl IWELLS. (0 => global''§ wells),
+                                      // for LGR IWELLS it refers to the LGWEL TAG.
+
             WTestRemaining = 45,       // Remaining number of times well can be tested (WTEST(4)).
                                        //   0 => Unlimited number of tests
                                        //  >0 => Well can be tested at most this many more times
@@ -415,6 +425,13 @@ namespace Opm { namespace RestartIO { namespace Helpers { namespace VectorItems
             ActionX  = 2, // ActionX name
         };
     } // ZWell
+
+    namespace LGWell {
+        enum index : std::vector<int>::size_type {
+            WellRef = 0, // Well Ref
+        };
+    } // LGWell
+
 }}}} // Opm::RestartIO::Helpers::VectorItems
 
 #endif // OPM_OUTPUT_ECLIPSE_VECTOR_WELL_HPP
