@@ -648,7 +648,7 @@ BOOST_AUTO_TEST_CASE_TEMPLATE(BlackOil, Evaluation, Types)
 
     const size_t numberOfRegions = FluidSystem::numRegions();
     for (size_t regionIndexToUse = 0; regionIndexToUse < numberOfRegions; ++regionIndexToUse) {
-        ParamCache paramCache(/*maxOilSat=*/0.5, regionIndexToUse);
+        ParamCache paramCache(regionIndexToUse);
 
         // create a parameter cache
         BOOST_CHECK_EQUAL(FluidSystem::reservoirTemperature(), fluidSystem.reservoirTemperature());
