@@ -115,6 +115,7 @@ void checkSystemHasRequiredDimensions( const UnitSystem& system) {
 
     BOOST_CHECK( system.hasDimension("Permeability"));
     BOOST_CHECK( system.hasDimension("Pressure"));
+    BOOST_CHECK( system.hasDimension("PressureDrop"));
     BOOST_CHECK( system.hasDimension("Temperature"));
 }
 
@@ -129,6 +130,7 @@ BOOST_AUTO_TEST_CASE(CreateMetricSystem) {
     BOOST_CHECK_EQUAL( Metric::Time         , system.getDimension("Time").getSIScaling() );
     BOOST_CHECK_EQUAL( Metric::Permeability , system.getDimension("Permeability").getSIScaling() );
     BOOST_CHECK_EQUAL( Metric::Pressure     , system.getDimension("Pressure").getSIScaling() );
+    BOOST_CHECK_EQUAL( Metric::PressureDrop , system.getDimension("PressureDrop").getSIScaling() );
 }
 
 
@@ -142,6 +144,7 @@ BOOST_AUTO_TEST_CASE(CreateFieldSystem) {
     BOOST_CHECK_EQUAL( Field::Time         , system.getDimension("Time").getSIScaling() );
     BOOST_CHECK_EQUAL( Field::Permeability , system.getDimension("Permeability").getSIScaling() );
     BOOST_CHECK_EQUAL( Field::Pressure     , system.getDimension("Pressure").getSIScaling() );
+    BOOST_CHECK_EQUAL( Field::PressureDrop , system.getDimension("PressureDrop").getSIScaling() );
 }
 
 
@@ -156,6 +159,7 @@ BOOST_AUTO_TEST_CASE(CreateInputSystem) {
     BOOST_CHECK_EQUAL( 1.0, system.getDimension("Time").getSIScaling() );
     BOOST_CHECK_EQUAL( 1.0, system.getDimension("Permeability").getSIScaling() );
     BOOST_CHECK_EQUAL( 1.0, system.getDimension("Pressure").getSIScaling() );
+    BOOST_CHECK_EQUAL( 1.0, system.getDimension("PressureDrop").getSIScaling() );
 
     BOOST_CHECK_EQUAL( static_cast<long int>(system.getType( )) , static_cast<long int>(UnitSystem::UnitType::UNIT_TYPE_INPUT) );
 }
@@ -312,11 +316,12 @@ BOOST_AUTO_TEST_CASE(METRIC_UNITS)
     BOOST_CHECK_CLOSE( metric.to_si( Meas::water_inverse_formation_volume_factor , 1.0 ) , 1.0 , 1.0e-10 );
     BOOST_CHECK_CLOSE( metric.to_si( Meas::liquid_productivity_index , 1.0 ) , 1.1574074074074073e-10 , 1.0e-10 );
     BOOST_CHECK_CLOSE( metric.to_si( Meas::gas_productivity_index , 1.0 ) , 1.1574074074074073e-10 , 1.0e-10 );
-    BOOST_CHECK_CLOSE( metric.to_si( Meas::energy, 1.0), 1000, 1e-10);
-    BOOST_CHECK_CLOSE( metric.to_si( Meas::icd_strength, 1.0), 7.46496e+14, 1e-10);
-    BOOST_CHECK_CLOSE( metric.to_si( Meas::gas_oil_ratio_rate, 1.0), 1.1574074074074073e-05, 1.0e-10 );
-    BOOST_CHECK_CLOSE( metric.to_si( Meas::moles, 1.0), 1000, 1.0e-10 );
-    BOOST_CHECK_CLOSE( metric.to_si( Meas::ppm, 1.0 ) , 1.0e-6 , 1.0e-10 );
+    BOOST_CHECK_CLOSE( metric.to_si( Meas::energy , 1.0 ) , 1000 , 1e-10 );
+    BOOST_CHECK_CLOSE( metric.to_si( Meas::icd_strength , 1.0 ) , 7.46496e+14 , 1e-10 );
+    BOOST_CHECK_CLOSE( metric.to_si( Meas::gas_oil_ratio_rate , 1.0 ) , 1.1574074074074073e-05 , 1.0e-10 );
+    BOOST_CHECK_CLOSE( metric.to_si( Meas::moles , 1.0 ) , 1000 , 1.0e-10 );
+    BOOST_CHECK_CLOSE( metric.to_si( Meas::ppm , 1.0 ) , 1.0e-6 , 1.0e-10 );
+    BOOST_CHECK_CLOSE( metric.to_si( Meas::thermalconductivity , 1.0 ) , 1.1574074074074073e-02 , 1.0e-10 );
 
     // ----------------------------------------------------------------
     // SI -> METRIC
@@ -354,11 +359,12 @@ BOOST_AUTO_TEST_CASE(METRIC_UNITS)
     BOOST_CHECK_CLOSE( metric.from_si( Meas::water_inverse_formation_volume_factor , 1.0 ) , 1.0 , 1.0e-10 );
     BOOST_CHECK_CLOSE( metric.from_si( Meas::liquid_productivity_index , 1.0 ) , 86.400e8 , 1.0e-10 );
     BOOST_CHECK_CLOSE( metric.from_si( Meas::gas_productivity_index , 1.0 ) , 86.400e8 , 1.0e-10 );
-    BOOST_CHECK_CLOSE( metric.from_si( Meas::energy, 1000.0), 1, 1e-10);
-    BOOST_CHECK_CLOSE( metric.from_si( Meas::icd_strength, 7.46496e+14), 1.0, 1e-10);
-    BOOST_CHECK_CLOSE( metric.from_si( Meas::gas_oil_ratio_rate, 1.0), 86.400e3, 1.0e-10 );
-    BOOST_CHECK_CLOSE( metric.from_si( Meas::moles, 1), 0.001, 1.0e-10 );
-    BOOST_CHECK_CLOSE( metric.from_si( Meas::ppm, 1.0 ) , 1.0e6 , 1.0e-10 );
+    BOOST_CHECK_CLOSE( metric.from_si( Meas::energy , 1000.0 ) , 1, 1e-10 );
+    BOOST_CHECK_CLOSE( metric.from_si( Meas::icd_strength , 7.46496e+14 ) , 1.0 , 1e-10 );
+    BOOST_CHECK_CLOSE( metric.from_si( Meas::gas_oil_ratio_rate , 1.0 ) , 86.400e3 , 1.0e-10 );
+    BOOST_CHECK_CLOSE( metric.from_si( Meas::moles , 1 ) , 0.001 , 1.0e-10 );
+    BOOST_CHECK_CLOSE( metric.from_si( Meas::ppm , 1.0 ) , 1.0e6 , 1.0e-10 );
+    BOOST_CHECK_CLOSE( metric.from_si( Meas::thermalconductivity , 1.1574074074074073e-02 ) , 1.0 , 1.0e-10 );
 
 }
 
@@ -408,9 +414,10 @@ BOOST_AUTO_TEST_CASE(FIELD_UNITS)
     BOOST_CHECK_CLOSE( field.to_si( Meas::gas_productivity_index , 1.0 ) , 3.277412800000001e-04 / 6.894757293168360e+03 , 1.0e-10 );
     BOOST_CHECK_CLOSE( field.to_si( Meas::energy , 1.0 ) , 1054.3503 , 1.0e-10 );
     BOOST_CHECK_CLOSE( field.to_si( Meas::icd_strength , 1.0 ) , 6.418842091749854e+16 , 1.0e-10 );
-    BOOST_CHECK_CLOSE( field.to_si( Meas::gas_oil_ratio_rate, 1.0), 1.1574074074074073e-05*178.1076066790352, 1.0e-10 );
-    BOOST_CHECK_CLOSE( field.to_si( Meas::moles, 1.0), 453.59237, 1.0e-5 );
+    BOOST_CHECK_CLOSE( field.to_si( Meas::gas_oil_ratio_rate , 1.0 ) , 1.1574074074074073e-05 * 178.1076066790352 , 1.0e-10 );
+    BOOST_CHECK_CLOSE( field.to_si( Meas::moles, 1.0 ) , 453.59237 , 1.0e-5 );
     BOOST_CHECK_CLOSE( field.to_si( Meas::ppm, 1.0 ) , 1.0e-6 , 1.0e-10 );
+    BOOST_CHECK_CLOSE( field.to_si( Meas::thermalconductivity , 1.0 ) , 7.2065719324146973e-02 , 1.0e-10 );
 
 
     // ----------------------------------------------------------------
@@ -450,9 +457,10 @@ BOOST_AUTO_TEST_CASE(FIELD_UNITS)
     BOOST_CHECK_CLOSE( field.from_si( Meas::gas_productivity_index , 1.0 ) , 3.051187204736614e+03 * 6.894757293168360e+03, 1.0e-10 );
     BOOST_CHECK_CLOSE( field.from_si( Meas::energy , 1054.3503 ) , 1.0 , 1.0e-10 );
     BOOST_CHECK_CLOSE( field.from_si( Meas::icd_strength , 6.418842091749854e+16 ) , 1.0 , 1.0e-10 );
-    BOOST_CHECK_CLOSE( field.from_si( Meas::gas_oil_ratio_rate, 1.0), 86.400e3*5.614583333333335e-03, 1.0e-10 );
-    BOOST_CHECK_CLOSE( field.from_si( Meas::moles, 1.0), 0.0022046226, 1.0e-5 );
-    BOOST_CHECK_CLOSE( field.from_si( Meas::ppm, 1.0 ) , 1.0e6 , 1.0e-10 );
+    BOOST_CHECK_CLOSE( field.from_si( Meas::gas_oil_ratio_rate , 1.0 ) , 86.400e3 * 5.614583333333335e-03 , 1.0e-10 );
+    BOOST_CHECK_CLOSE( field.from_si( Meas::moles , 1.0 ) , 0.0022046226 , 1.0e-5 );
+    BOOST_CHECK_CLOSE( field.from_si( Meas::ppm , 1.0 ) , 1.0e6 , 1.0e-10 );
+    BOOST_CHECK_CLOSE( field.from_si( Meas::thermalconductivity , 7.2065719324146973e-02 ) , 1.0 , 1.0e-10 );
 
 }
 
@@ -502,9 +510,10 @@ BOOST_AUTO_TEST_CASE(LAB_UNITS)
     BOOST_CHECK_CLOSE( lab.to_si( Meas::gas_productivity_index , 1.0 ) , 2.777777777777778e-10 / 101325.0 , 1.0e-10 );
     BOOST_CHECK_CLOSE( lab.to_si( Meas::energy , 1.0 ) , 1.0 , 1.0e-10 );
     BOOST_CHECK_CLOSE( lab.to_si( Meas::icd_strength , 1.0 ) , 1.313172e+24 , 1.0e-10 );
-    BOOST_CHECK_CLOSE( lab.to_si( Meas::gas_oil_ratio_rate, 1.0), 2.777777777777778e-4, 1.0e-10 );
-    BOOST_CHECK_CLOSE( lab.to_si( Meas::moles, 1), 1, 1.0e-10 );
-    BOOST_CHECK_CLOSE( lab.to_si( Meas::ppm, 1.0 ) , 1.0e-6 , 1.0e-10 );
+    BOOST_CHECK_CLOSE( lab.to_si( Meas::gas_oil_ratio_rate , 1.0 ) , 2.777777777777778e-4 , 1.0e-10 );
+    BOOST_CHECK_CLOSE( lab.to_si( Meas::moles , 1 ) , 1, 1.0e-10 );
+    BOOST_CHECK_CLOSE( lab.to_si( Meas::ppm , 1.0 ) , 1.0e-6 , 1.0e-10 );
+    BOOST_CHECK_CLOSE( lab.to_si( Meas::thermalconductivity , 1.0 ) , 2.777777777777778e-02 , 1.0e-10 );
 
 
     // ----------------------------------------------------------------
@@ -545,9 +554,10 @@ BOOST_AUTO_TEST_CASE(LAB_UNITS)
     BOOST_CHECK_CLOSE( lab.from_si( Meas::gas_productivity_index , 1.0 ) , 3.6e9 * 101325.0 , 1.0e-10 );
     BOOST_CHECK_CLOSE( lab.from_si( Meas::energy , 1.0 ) , 1.0 , 1.0e-10 );
     BOOST_CHECK_CLOSE( lab.from_si( Meas::icd_strength , 1.0 ) , 7.615148662932201e-25 , 1.0e-10 );
-    BOOST_CHECK_CLOSE( lab.from_si( Meas::gas_oil_ratio_rate, 1.0), 3.6e3, 1.0e-10 );
-    BOOST_CHECK_CLOSE( lab.from_si( Meas::moles, 1), 1, 1.0e-10 );
-    BOOST_CHECK_CLOSE( lab.from_si( Meas::ppm, 1.0 ) , 1.0e6 , 1.0e-10 );
+    BOOST_CHECK_CLOSE( lab.from_si( Meas::gas_oil_ratio_rate , 1.0 ) , 3.6e3, 1.0e-10 );
+    BOOST_CHECK_CLOSE( lab.from_si( Meas::moles , 1 ) , 1, 1.0e-10 );
+    BOOST_CHECK_CLOSE( lab.from_si( Meas::ppm , 1.0 ) , 1.0e6 , 1.0e-10 );
+    BOOST_CHECK_CLOSE( lab.from_si( Meas::thermalconductivity , 2.777777777777778e-02 ) , 1.0 , 1.0e-10 );
 
 }
 
@@ -597,9 +607,10 @@ BOOST_AUTO_TEST_CASE(PVT_M_UNITS)
     BOOST_CHECK_CLOSE( pvt_m.to_si( Meas::gas_productivity_index , 1.0 ) , 1.1574074074074073e-05 /101325.0 , 1.0e-10 );
     BOOST_CHECK_CLOSE( pvt_m.to_si( Meas::energy , 1.0 ) , 1.0e3 , 1.0e-10 );
     BOOST_CHECK_CLOSE( pvt_m.to_si( Meas::icd_strength , 1.0 ) , 7.56387072e+14 , 1.0e-10 );
-    BOOST_CHECK_CLOSE( pvt_m.to_si( Meas::gas_oil_ratio_rate, 1.0), 1.1574074074074073e-05, 1.0e-10 );
-    BOOST_CHECK_CLOSE( pvt_m.to_si( Meas::moles, 1), 1000, 1.0e-10 );
-    BOOST_CHECK_CLOSE( pvt_m.to_si( Meas::ppm, 1.0 ) , 1.0e-6 , 1.0e-10 );
+    BOOST_CHECK_CLOSE( pvt_m.to_si( Meas::gas_oil_ratio_rate , 1.0 ), 1.1574074074074073e-05 , 1.0e-10 );
+    BOOST_CHECK_CLOSE( pvt_m.to_si( Meas::moles , 1 ) , 1000 , 1.0e-10 );
+    BOOST_CHECK_CLOSE( pvt_m.to_si( Meas::ppm , 1.0 ) , 1.0e-6 , 1.0e-10 );
+    BOOST_CHECK_CLOSE( pvt_m.to_si( Meas::thermalconductivity , 1.0 ) , 1.1574074074074073e-02 , 1.0e-10 );
 
 
     // ----------------------------------------------------------------
@@ -640,9 +651,10 @@ BOOST_AUTO_TEST_CASE(PVT_M_UNITS)
     BOOST_CHECK_CLOSE( pvt_m.from_si( Meas::gas_productivity_index , 1.0 ) , 86.400e3 * 101325.0, 1.0e-10 );
     BOOST_CHECK_CLOSE( pvt_m.from_si( Meas::energy , 1.0 ) , 1.0e-3 , 1.0e-10 );
     BOOST_CHECK_CLOSE( pvt_m.from_si( Meas::icd_strength , 1.0 ) , 1.322074420647951e-15 , 1.0e-10 );
-    BOOST_CHECK_CLOSE( pvt_m.from_si( Meas::gas_oil_ratio_rate, 1.0), 86.400e3, 1.0e-10 );
-    BOOST_CHECK_CLOSE( pvt_m.from_si( Meas::moles, 1), 0.001, 1.0e-10 );
-    BOOST_CHECK_CLOSE( pvt_m.from_si( Meas::ppm, 1), 1.e6, 1.0e-10 );
+    BOOST_CHECK_CLOSE( pvt_m.from_si( Meas::gas_oil_ratio_rate , 1.0 ) , 86.400e3 , 1.0e-10 );
+    BOOST_CHECK_CLOSE( pvt_m.from_si( Meas::moles , 1 ) , 0.001 , 1.0e-10 );
+    BOOST_CHECK_CLOSE( pvt_m.from_si( Meas::ppm , 1) , 1.e6 , 1.0e-10 );
+    BOOST_CHECK_CLOSE( pvt_m.from_si( Meas::thermalconductivity , 1.1574074074074073e-02 ) , 1.0 , 1.0e-10 );
 
 }
 
