@@ -24,6 +24,7 @@
 
 #include <opm/common/utility/OpmInputError.hpp>
 
+#include <opm/input/eclipse/EclipseState/Aquifer/NumericalAquifer/NumericalAquifers.hpp>
 #include <opm/input/eclipse/EclipseState/EclipseState.hpp>
 #include <opm/input/eclipse/EclipseState/Grid/FieldPropsManager.hpp>
 #include <opm/input/eclipse/EclipseState/Runspec.hpp>
@@ -50,6 +51,12 @@
 #include <opm/input/eclipse/Parser/ParseContext.hpp>
 #include <opm/input/eclipse/Parser/Parser.hpp>
 
+#include <cstddef>
+#include <memory>
+#include <stdexcept>
+#include <string>
+#include <vector>
+
 using namespace Opm;
 
 namespace {
@@ -236,14 +243,13 @@ SGCWMIS
 
 BOOST_AUTO_TEST_CASE( SORWMIS ) {
 
-    Parser parser;
-    // missing miscible keyword
-    BOOST_CHECK_THROW (parser.parseString(sorwmisData), OpmInputError );
+    // Missing miscible keyword
+    BOOST_CHECK_THROW(Parser{}.parseString(sorwmisData), OpmInputError);
 
-    //too many tables
-    BOOST_CHECK_THROW( parser.parseString(miscibleTightData + sorwmisData), OpmInputError);
+    // Too many tables
+    BOOST_CHECK_THROW(Parser{}.parseString(miscibleTightData + sorwmisData), OpmInputError);
 
-    auto deck1 =  parser.parseString(miscibleData + sorwmisData);
+    const auto deck1 = Parser{}.parseString(miscibleData + sorwmisData);
 
     const auto& sorwmis = deck1["SORWMIS"].back();
     const auto& miscible = deck1["MISCIBLE"].back();
@@ -253,9 +259,9 @@ BOOST_AUTO_TEST_CASE( SORWMIS ) {
     const auto& sorwmis1 = sorwmis.getRecord(1);
 
     // test number of columns
-    size_t ntmisc = miscible0.getItem(0).get< int >(0);
-    Opm::SorwmisTable sorwmisTable0(sorwmis0.getItem(0), 0);
-    BOOST_CHECK_EQUAL(sorwmisTable0.numColumns(),ntmisc);
+    const std::size_t ntmisc = miscible0.getItem(0).get<int>(0);
+    const Opm::SorwmisTable sorwmisTable0(sorwmis0.getItem(0), 0);
+    BOOST_CHECK_EQUAL(sorwmisTable0.numColumns(), ntmisc);
 
     // test table input 1
     BOOST_CHECK_EQUAL(3U, sorwmisTable0.getWaterSaturationColumn().size());
@@ -263,7 +269,7 @@ BOOST_AUTO_TEST_CASE( SORWMIS ) {
     BOOST_CHECK_EQUAL(0.0, sorwmisTable0.getMiscibleResidualOilColumn()[2]);
 
     // test table input 2
-    Opm::SorwmisTable sorwmisTable1(sorwmis1.getItem(0), 1);
+    const Opm::SorwmisTable sorwmisTable1(sorwmis1.getItem(0), 1);
     BOOST_CHECK_EQUAL(sorwmisTable1.numColumns(),ntmisc);
 
     BOOST_CHECK_EQUAL(3U, sorwmisTable1.getWaterSaturationColumn().size());
@@ -616,9 +622,8 @@ BOOST_AUTO_TEST_CASE( MULTISEGMENT_ABS ) {
 
     const EclipseState state(deck);
     const TableManager table ( deck );
-    Runspec runspec (deck);
-    auto python = std::make_shared<Python>();
-    const Schedule sched(deck, state, python);
+    const Runspec runspec (deck);
+    const Schedule sched(deck, state, std::make_shared<Python>());
 
     // checking the relation between segments and completions
     // and also the depth of completions
@@ -953,9 +958,6 @@ PVTO
     BOOST_CHECK_EQUAL( BO.back( ) , 1.01 );
 
     BOOST_CHECK_CLOSE(1.15 , table0.evaluate( "BO" , 250*1e5 ) , 1e-6);
-
-    BOOST_CHECK_CLOSE( 1.15 , pvtoTable.evaluate( "BO" , 1e-3 , 250*1e5 ) , 1e-6 );
-    BOOST_CHECK_CLOSE( 1.15 , pvtoTable.evaluate( "BO" , 0.0 , 250*1e5 ) , 1e-6 );
 }
 
 BOOST_AUTO_TEST_CASE( SGOF ) {
@@ -1393,15 +1395,16 @@ WOPR
 }
 
 BOOST_AUTO_TEST_CASE( WCONPROD ) {
-    Parser parser;
-    std::string wconprodFile(pathprefix() + "WellWithWildcards/WCONPROD1");
-    auto deck =  parser.parseFile(wconprodFile);
+    const std::string wconprodFile(pathprefix() + "WellWithWildcards/WCONPROD1");
+    const auto deck = Parser{}.parseFile(wconprodFile);
     EclipseGrid grid(30,30,30);
-    TableManager table ( deck );
-    FieldPropsManager fp( deck, Phases{true, true, true}, grid, table);
-    Runspec runspec (deck);
-    auto python = std::make_shared<Python>();
-    Schedule sched(deck, grid, fp, runspec, python);
+    const TableManager table ( deck );
+    const FieldPropsManager fp( deck, Phases{true, true, true}, grid, table);
+    const Runspec runspec (deck);
+    const Schedule sched {
+        deck, grid, fp, NumericalAquifers{},
+        runspec, std::make_shared<Python>()
+    };
 
     BOOST_CHECK_EQUAL(5U, sched.numWells());
     BOOST_CHECK(sched.hasWell("INJE1"));
@@ -1422,9 +1425,9 @@ BOOST_AUTO_TEST_CASE( WCONPROD ) {
     {
         const auto& well0 = sched.getWell("PROD3", 0 );
         const auto& well1 = sched.getWell("PROD3", 1 );
-        BOOST_CHECK_CLOSE(0   , well0.getProductionProperties().OilRate.get<double>(), 0.001);
+        BOOST_CHECK_THROW(well0.getProductionProperties().OilRate.get<double>(), std::invalid_argument);
         BOOST_CHECK_CLOSE(1500, well1.getProductionProperties().OilRate.get<double>(), 0.001);
-        BOOST_CHECK_CLOSE(0/Metric::Time   , well0.getProductionProperties().OilRate.getSI(), 0.001);
+        BOOST_CHECK_THROW(well0.getProductionProperties().OilRate.getSI(), std::invalid_argument);
         BOOST_CHECK_CLOSE(1500/Metric::Time, well1.getProductionProperties().OilRate.getSI(), 0.001);
     }
 
@@ -1447,7 +1450,11 @@ BOOST_AUTO_TEST_CASE( WCONINJE )
     const TableManager table (deck);
     const FieldPropsManager fp(deck, Phases{true, true, true}, grid, table);
     const Runspec runspec(deck);
-    const Schedule sched(deck, grid, fp, runspec, std::make_shared<Python>());
+    const Schedule sched {
+        deck, grid, fp, NumericalAquifers{},
+        runspec, std::make_shared<Python>()
+    };
+
     SummaryState st(TimeService::now(), runspec.udqParams().undefinedValue());
 
     BOOST_CHECK_EQUAL(5U, sched.numWells());
